@@ -18,13 +18,12 @@
     along with this program; if not, write to the Free Software 
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: time.h 948 2006-05-15 17:12:44Z pbd $
 */
 
 #ifndef __jack_time_h__
 #define __jack_time_h__
 
-#include <jack/internal.h>
+#include <jack/types.h>
 
 extern jack_time_t (*_jack_get_microseconds)(void);
 
