@@ -16,7 +16,6 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: alsa_driver.c 960 2006-05-25 14:01:10Z paul $
 */
 
 #include <math.h>
@@ -415,14 +414,14 @@ alsa_driver_configure_stream (alsa_driver_t *driver, char *device_name,
 		}
 	}
 	
-	format = (sample_width == 4) ? 0 : FIRST_16BIT_FORMAT;
+	format = (sample_width == 4) ? 0 : NUMFORMATS - 1;
 
 	while (1) {
 		if ((err = snd_pcm_hw_params_set_format (
 			     handle, hw_params, formats[format].format)) < 0) {
 
 			if ((sample_width == 4
-			     ? format++ >= FIRST_16BIT_FORMAT
+			     ? format++ >= NUMFORMATS - 1
 			     : format-- <= 0)) {
 				jack_error ("Sorry. The audio interface \"%s\""
 					    " doesn't support any of the"
@@ -623,8 +622,8 @@ alsa_driver_set_parameters (alsa_driver_t *driver,
 	driver->user_nperiods = user_nperiods;
 
 	fprintf (stderr, "configuring for %" PRIu32 "Hz, period = %"
-		 PRIu32 " frames, buffer = %" PRIu32 " periods\n",
-		 rate, frames_per_cycle, user_nperiods);
+		 PRIu32 " frames (%.1f ms), buffer = %" PRIu32 " periods\n",
+		 rate, frames_per_cycle, (((float)frames_per_cycle / (float) rate) * 1000.0f), user_nperiods);
 	
 	if (driver->capture_handle) {
 		if (alsa_driver_configure_stream (
@@ -1023,6 +1022,9 @@ alsa_driver_start (alsa_driver_t *driver)
 		malloc (sizeof (struct pollfd) * 
 			(driver->playback_nfds + driver->capture_nfds + 2));
 
+	if (driver->midi && !driver->xrun_recovery)
+		(driver->midi->start)(driver->midi);
+
 	if (driver->playback_handle) {
 		/* fill playback buffer with zeroes, and mark 
 		   all fragments as having data.
@@ -1127,15 +1129,26 @@ alsa_driver_stop (alsa_driver_t *driver)
 		driver->hw->set_input_monitor_mask (driver->hw, 0);
 	}
 
+	if (driver->midi && !driver->xrun_recovery)
+		(driver->midi->stop)(driver->midi);
+
 	return 0;
 }
 
 static int
 alsa_driver_restart (alsa_driver_t *driver)
 {
- 	if (driver->nt_stop((struct _jack_driver_nt *) driver))
- 	    return -1;
- 	return driver->nt_start((struct _jack_driver_nt *) driver);
+	int res;
+
+	driver->xrun_recovery = 1;
+ 	if ((res = driver->nt_stop((struct _jack_driver_nt *) driver))==0)
+		res = driver->nt_start((struct _jack_driver_nt *) driver);
+	driver->xrun_recovery = 0;
+
+	if (res && driver->midi)
+		(driver->midi->stop)(driver->midi);
+
+	return res;
 }
 
 static int
@@ -1235,9 +1248,10 @@ alsa_driver_wait (alsa_driver_t *driver, int extra_fd, int *status, float
 	
 	while (need_playback || need_capture) {
 
-		unsigned int p_timed_out, c_timed_out;
+		int poll_result;
 		unsigned int ci = 0;
 		unsigned int nfds;
+		unsigned short revents;
 
 		nfds = 0;
 
@@ -1281,7 +1295,8 @@ alsa_driver_wait (alsa_driver_t *driver, int extra_fd, int *status, float
 			driver->poll_late++;
 		}
 
-		if (poll (driver->pfd, nfds, driver->poll_timeout) < 0) {
+		poll_result = poll (driver->pfd, nfds, driver->poll_timeout);
+		if (poll_result < 0) {
 
 			if (errno == EINTR) {
 				printf ("poll interrupt\n");
@@ -1337,25 +1352,20 @@ alsa_driver_wait (alsa_driver_t *driver, int extra_fd, int *status, float
 			return (driver->pfd[nfds-1].revents == POLLIN) ? 0 : -1;
 		}
 
-		p_timed_out = 0;
-		
 		if (need_playback) {
-			for (i = 0; i < driver->playback_nfds; i++) {
-				if (driver->pfd[i].revents & POLLERR) {
-					xrun_detected = TRUE;
-				}
-				
-				if (driver->pfd[i].revents == 0) {
-					p_timed_out++;
-#ifdef DEBUG_WAKEUP
-					fprintf (stderr, "%" PRIu64
-						 " playback stream timed out\n",
-						 poll_ret);
-#endif
-				}
+			if (snd_pcm_poll_descriptors_revents
+			    (driver->playback_handle, &driver->pfd[0],
+			     driver->playback_nfds, &revents) < 0) {
+				jack_error ("ALSA: playback revents failed");
+				*status = -6;
+				return 0;
 			}
-			
-			if (p_timed_out == 0) {
+
+			if (revents & POLLERR) {
+				xrun_detected = TRUE;
+			}
+
+			if (revents & POLLOUT) {
 				need_playback = 0;
 #ifdef DEBUG_WAKEUP
 				fprintf (stderr, "%" PRIu64
@@ -1364,26 +1374,21 @@ alsa_driver_wait (alsa_driver_t *driver, int extra_fd, int *status, float
 #endif
 			}
 		}
-		
-		c_timed_out = 0;
 
 		if (need_capture) {
-			for (i = ci; i < nfds; i++) {
-				if (driver->pfd[i].revents & POLLERR) {
-					xrun_detected = TRUE;
-				}
-				
-				if (driver->pfd[i].revents == 0) {
-					c_timed_out++;
-#ifdef DEBUG_WAKEUP
-					fprintf (stderr, "%" PRIu64
-						 " capture stream timed out\n",
-						 poll_ret);
-#endif
-				}
+			if (snd_pcm_poll_descriptors_revents
+			    (driver->capture_handle, &driver->pfd[ci],
+			     driver->capture_nfds, &revents) < 0) {
+				jack_error ("ALSA: capture revents failed");
+				*status = -6;
+				return 0;
 			}
-			
-			if (c_timed_out == 0) {
+
+			if (revents & POLLERR) {
+				xrun_detected = TRUE;
+			}
+
+			if (revents & POLLIN) {
 				need_capture = 0;
 #ifdef DEBUG_WAKEUP
 				fprintf (stderr, "%" PRIu64
@@ -1393,8 +1398,7 @@ alsa_driver_wait (alsa_driver_t *driver, int extra_fd, int *status, float
 			}
 		}
 		
-		if ((p_timed_out && (p_timed_out == driver->playback_nfds)) &&
-		    (c_timed_out && (c_timed_out == driver->capture_nfds))){
+		if (poll_result == 0) {
 			jack_error ("ALSA: poll time out, polled for %" PRIu64
 				    " usecs",
 				    poll_ret - poll_enter);
@@ -1548,13 +1552,21 @@ alsa_driver_read (alsa_driver_t *driver, jack_nframes_t nframes)
 	jack_port_t* port;
 	int err;
 
-	if (!driver->capture_handle || driver->engine->freewheeling) {
-		return 0;
-	}
 	if (nframes > driver->frames_per_cycle) {
 		return -1;
 	}
 	
+	if (driver->engine->freewheeling) {
+		return 0;
+	}
+
+	if (driver->midi)
+		(driver->midi->read)(driver->midi, nframes);
+	
+	if (!driver->capture_handle) {
+		return 0;
+	}
+
 	nread = 0;
 	contiguous = 0;
 	orig_nframes = nframes;
@@ -1623,6 +1635,9 @@ alsa_driver_write (alsa_driver_t* driver, jack_nframes_t nframes)
 		return -1;
 	}
 
+	if (driver->midi)
+		(driver->midi->write)(driver->midi, nframes);
+	
 	nwritten = 0;
 	contiguous = 0;
 	orig_nframes = nframes;
@@ -1752,7 +1767,7 @@ alsa_driver_attach (alsa_driver_t *driver)
 
 	for (chn = 0; chn < driver->capture_nchannels; chn++) {
 
-		snprintf (buf, sizeof(buf) - 1, "capture_%lu", chn+1);
+		snprintf (buf, sizeof(buf), "capture_%lu", chn+1);
 
 		if ((port = jack_port_register (driver->client, buf,
 						JACK_DEFAULT_AUDIO_TYPE,
@@ -1806,6 +1821,12 @@ alsa_driver_attach (alsa_driver_t *driver)
 		}
 	}
 
+	if (driver->midi) {
+		int err = (driver->midi->attach)(driver->midi);
+		if (err)
+			jack_error("ALSA: cannot attach midi: %d", err);
+	}
+	
 
 	return jack_activate (driver->client);
 }
@@ -1819,6 +1840,9 @@ alsa_driver_detach (alsa_driver_t *driver)
 		return 0;
 	}
 
+	if (driver->midi)
+		(driver->midi->detach)(driver->midi);
+	
 	for (node = driver->capture_ports; node;
 	     node = jack_slist_next (node)) {
 		jack_port_unregister (driver->client,
@@ -1905,6 +1929,9 @@ alsa_driver_delete (alsa_driver_t *driver)
 {
 	JSList *node;
 
+	if (driver->midi)
+		(driver->midi->destroy)(driver->midi);
+
 	for (node = driver->clock_sync_listeners; node;
 	     node = jack_slist_next (node)) {
 		free (node->data);
@@ -1976,7 +2003,8 @@ alsa_driver_new (char *name, char *playback_alsa_device,
 		 int user_playback_nchnls,
 		 int shorts_first,
 		 jack_nframes_t capture_latency,
-		 jack_nframes_t playback_latency
+		 jack_nframes_t playback_latency,
+		 alsa_midi_t *midi_driver
 		 )
 {
 	int err;
@@ -2059,6 +2087,9 @@ alsa_driver_new (char *name, char *playback_alsa_device,
 	driver->alsa_name_playback = strdup (playback_alsa_device);
 	driver->alsa_name_capture = strdup (capture_alsa_device);
 
+	driver->midi = midi_driver;
+	driver->xrun_recovery = 0;
+
 	if (alsa_driver_check_card_type (driver)) {
 		alsa_driver_delete (driver);
 		return NULL;
@@ -2220,8 +2251,8 @@ alsa_driver_new (char *name, char *playback_alsa_device,
 	driver->capture_and_playback_not_synced = FALSE;
 
 	if (driver->capture_handle && driver->playback_handle) {
-		if (snd_pcm_link (driver->capture_handle,
-				  driver->playback_handle) != 0) {
+		if (snd_pcm_link (driver->playback_handle,
+				  driver->capture_handle) != 0) {
 			driver->capture_and_playback_not_synced = TRUE;
 		} 
 	}
@@ -2333,7 +2364,7 @@ driver_get_descriptor ()
 	desc = calloc (1, sizeof (jack_driver_desc_t));
 
 	strcpy (desc->name,"alsa");
-	desc->nparams = 17;
+	desc->nparams = 18;
   
 	params = calloc (desc->nparams, sizeof (jack_driver_param_desc_t));
 
@@ -2484,6 +2515,18 @@ driver_get_descriptor ()
 	strcpy (params[i].short_desc, "Extra output latency (frames)");
 	strcpy (params[i].long_desc, params[i].short_desc);
 
+	i++;
+	strcpy (params[i].name, "midi");
+	params[i].character  = 'X';
+	params[i].type       = JackDriverParamString;
+	strcpy (params[i].value.str,  "none");
+	strcpy (params[i].short_desc, "ALSA MIDI driver (seq|raw)");
+	strcpy (params[i].long_desc,
+		"ALSA MIDI driver:\n"
+		" none - no MIDI driver\n"
+		" seq - ALSA Sequencer driver\n"
+		" raw - ALSA RawMIDI driver\n");
+
 	desc->params = params;
 
 	return desc;
@@ -2509,6 +2552,8 @@ driver_initialize (jack_client_t *client, const JSList * params)
 	int shorts_first = FALSE;
 	jack_nframes_t systemic_input_latency = 0;
 	jack_nframes_t systemic_output_latency = 0;
+	char *midi_driver_name = "none";
+	alsa_midi_t *midi = NULL;
 	const JSList * node;
 	const jack_driver_param_t * param;
 
@@ -2597,6 +2642,10 @@ driver_initialize (jack_client_t *client, const JSList * params)
 			systemic_output_latency = param->value.ui;
 			break;
 
+		case 'X':
+			midi_driver_name = strdup (param->value.str);
+			break;
+
 		}
 	}
 			
@@ -2606,6 +2655,12 @@ driver_initialize (jack_client_t *client, const JSList * params)
 		playback = TRUE;
 	}
 
+	if (strcmp(midi_driver_name, "seq")==0) {
+		midi = alsa_seqmidi_new(client, NULL);
+	} else if (strcmp(midi_driver_name, "raw")==0) {
+		midi = alsa_rawmidi_new(client);
+	}
+
 	return alsa_driver_new ("alsa_pcm", playback_pcm_name,
 				capture_pcm_name, client,
 				frames_per_interrupt, 
@@ -2615,7 +2670,7 @@ driver_initialize (jack_client_t *client, const JSList * params)
 				user_capture_nchnls, user_playback_nchnls,
 				shorts_first, 
 				systemic_input_latency,
-				systemic_output_latency);
+				systemic_output_latency, midi);
 }
 
 void
