@@ -16,7 +16,6 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: usx2y.c 999 2006-11-29 20:06:36Z rncbc $
 */
 
 #include <jack/hardware.h>
@@ -230,6 +229,9 @@ usx2y_driver_start (alsa_driver_t *driver)
 		return -1;
 	}
 
+	if (driver->midi && !driver->xrun_recovery)
+		(driver->midi->start)(driver->midi);
+
 	if (driver->playback_handle) {
 /* 		int i, j; */
 /* 		char buffer[2000]; */
@@ -380,6 +382,9 @@ usx2y_driver_stop (alsa_driver_t *driver)
 
 	munmap(h->hwdep_pcm_shm, sizeof(snd_usX2Y_hwdep_pcm_shm_t));
 
+	if (driver->midi && !driver->xrun_recovery)
+		(driver->midi->stop)(driver->midi);
+
 	return 0;
 }
 
@@ -482,6 +487,9 @@ usx2y_driver_read (alsa_driver_t *driver, jack_nframes_t nframes)
 		return 0;
 	}
 
+    if (driver->midi)
+        (driver->midi->read)(driver->midi, nframes);
+
 	nread = 0;
 
 	if (snd_pcm_mmap_begin (driver->capture_handle,
@@ -556,6 +564,9 @@ usx2y_driver_write (alsa_driver_t* driver, jack_nframes_t nframes)
 		return 0;
 	}
 
+    if (driver->midi)
+        (driver->midi->write)(driver->midi, nframes);
+
 	nwritten = 0;
 
 	/* check current input monitor request status */
