@@ -17,7 +17,6 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: dummy_driver.c 985 2006-08-29 15:21:17Z paul $
 */
 
 #include <math.h>
@@ -44,7 +43,7 @@
 void
 FakeVideoSync( dummy_driver_t *driver )
 {
-        static const int VIDEO_SYNC_PERIOD = 48000 / 30;
+        #define VIDEO_SYNC_PERIOD (48000 / 30)
         static int vidCounter = VIDEO_SYNC_PERIOD;
         
         int period = driver->period_size;
