@@ -1,7 +1,7 @@
 /*
 
 	OSS driver for Jack
-	Copyright (C) 2003-2005 Jussi Laako <jussi@sonarnerd.net>
+	Copyright (C) 2003-2007 Jussi Laako <jussi@sonarnerd.net>
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -66,6 +66,15 @@
 #include "oss_driver.h"
 
 
+#ifndef SNDCTL_DSP_COOKEDMODE
+#ifdef _SIOWR
+#define SNDCTL_DSP_COOKEDMODE _SIOWR('P', 30, int)
+#else  /* _SIOWR */
+#warning "Unable to define cooked mode!"
+#define OSS_NO_COOKED_MODE
+#endif  /* _SIOWR */
+#endif  /* SNDCTL_DSP_COOKEDMODE */
+
 #define OSS_DRIVER_N_PARAMS	11
 const static jack_driver_param_desc_t oss_params[OSS_DRIVER_N_PARAMS] = {
 	{ "rate",
@@ -431,6 +440,7 @@ static int oss_driver_detach (oss_driver_t *driver, jack_engine_t *engine)
 
 static int oss_driver_start (oss_driver_t *driver)
 {
+	int flags = 0;
 	int format;
 	int channels;
 	int samplerate;
@@ -456,17 +466,21 @@ static int oss_driver_start (oss_driver_t *driver)
 			samplesize = sizeof(short);
 			break;
 	}
+	driver->trigger = 0;
 	if (strcmp(indev, outdev) != 0)
 	{
 		if (driver->capture_channels > 0)
 		{
-			infd = open(indev, O_RDONLY);
+			infd = open(indev, O_RDONLY|O_EXCL);
 			if (infd < 0)
 			{
 				jack_error(
 					"OSS: failed to open input device %s: %s@%i, errno=%d",
 					indev, __FILE__, __LINE__, errno);
 			}
+#ifndef OSS_NO_COOKED_MODE
+			ioctl(infd, SNDCTL_DSP_COOKEDMODE, &flags);
+#endif
 			fragsize = driver->period_size * 
 				driver->capture_channels * samplesize;
 			set_fragment(infd, fragsize, driver->nperiods);
@@ -475,13 +489,16 @@ static int oss_driver_start (oss_driver_t *driver)
 
 		if (driver->playback_channels > 0)
 		{
-			outfd = open(outdev, O_WRONLY);
+			outfd = open(outdev, O_WRONLY|O_EXCL);
 			if (outfd < 0)
 			{
 				jack_error(
 					"OSS: failed to open output device %s: %s@%i, errno=%d",
 					outdev, __FILE__, __LINE__, errno);
 			}
+#ifndef OSS_NO_COOKED_MODE
+			ioctl(outfd, SNDCTL_DSP_COOKEDMODE, &flags);
+#endif
 			fragsize = driver->period_size * 
 				driver->playback_channels * samplesize;
 			set_fragment(outfd, fragsize, driver->nperiods);
@@ -493,7 +510,7 @@ static int oss_driver_start (oss_driver_t *driver)
 		if (driver->capture_channels != 0 &&
 			driver->playback_channels == 0)
 		{
-			infd = open(indev, O_RDWR);
+			infd = open(indev, O_RDWR|O_EXCL);
 			outfd = -1;
 			if (infd < 0)
 			{
@@ -502,12 +519,15 @@ static int oss_driver_start (oss_driver_t *driver)
 					indev, __FILE__, __LINE__, errno);
 				return -1;
 			}
+#ifndef OSS_NO_COOKED_MODE
+			ioctl(infd, SNDCTL_DSP_COOKEDMODE, &flags);
+#endif
 		}
 		else if (driver->capture_channels == 0 &&
 			driver->playback_channels != 0)
 		{
 			infd = -1;
-			outfd = open(outdev, O_RDWR);
+			outfd = open(outdev, O_RDWR|O_EXCL);
 			if (outfd < 0)
 			{
 				jack_error(
@@ -515,10 +535,13 @@ static int oss_driver_start (oss_driver_t *driver)
 					outdev, __FILE__, __LINE__, errno);
 				return -1;
 			}
+#ifndef OSS_NO_COOKED_MODE
+			ioctl(outfd, SNDCTL_DSP_COOKEDMODE, &flags);
+#endif
 		}
 		else
 		{
-			infd = outfd = open(indev, O_RDWR);
+			infd = outfd = open(indev, O_RDWR|O_EXCL);
 			if (infd < 0)
 			{
 				jack_error(
@@ -526,9 +549,14 @@ static int oss_driver_start (oss_driver_t *driver)
 					indev, __FILE__, __LINE__, errno);
 				return -1;
 			}
+#ifndef OSS_NO_COOKED_MODE
+			ioctl(infd, SNDCTL_DSP_COOKEDMODE, &flags);
+#endif
 		}
 		if (infd >= 0 && outfd >= 0)
 		{
+			ioctl(outfd, SNDCTL_DSP_SETTRIGGER, &driver->trigger);
+			driver->trigger = (PCM_ENABLE_INPUT|PCM_ENABLE_OUTPUT);
 			if (ioctl(infd, SNDCTL_DSP_SETDUPLEX, 0) < 0)
 			{
 				if (errno != EINVAL) /* Dont care */
@@ -961,6 +989,13 @@ static void *io_thread (void *param)
 				__FILE__, __LINE__);
 			return NULL;
 		}
+		if (driver->trigger)
+		{
+			/* don't care too much if this fails */
+			memset(localbuf, 0x00, localsize);
+			write(driver->outfd, localbuf, localsize);
+			ioctl(driver->outfd, SNDCTL_DSP_SETTRIGGER, &driver->trigger);
+		}
 
 		while (driver->run)
 		{
@@ -992,6 +1027,13 @@ static void *io_thread (void *param)
 		jack_error("OSS: malloc() failed: %s@%i", __FILE__, __LINE__);
 		return NULL;
 	}
+	if (driver->trigger)
+	{
+		/* don't care too much if this fails */
+		memset(localbuf, 0x00, localsize);
+		write(driver->outfd, localbuf, driver->outdevbufsize);
+		ioctl(driver->outfd, SNDCTL_DSP_SETTRIGGER, &driver->trigger);
+	}
 
 	while (driver->run)
 	{
@@ -1119,6 +1161,7 @@ jack_driver_t * driver_initialize (jack_client_t *client,
 	driver->indev = NULL;
 	driver->outdev = NULL;
 	driver->ignorehwbuf = 0;
+	driver->trigger = 0;
 
 	pnode = params;
 	while (pnode != NULL)
