@@ -21,9 +21,11 @@ void
 show_usage (void)
 {
 	show_version ();
-	fprintf (stderr, "\nUsage: %s [options]\n", my_name);
-	fprintf (stderr, "List active Jack ports, and optionally display extra information.\n\n");
+	fprintf (stderr, "\nUsage: %s [options] [filter string]\n", my_name);
+	fprintf (stderr, "List active Jack ports, and optionally display extra information.\n");
+	fprintf (stderr, "Optionally filter ports which match ALL strings provided after any options.\n\n");
 	fprintf (stderr, "Display options:\n");
+	fprintf (stderr, "        -A, --aliases         List aliases for each port\n");
 	fprintf (stderr, "        -c, --connections     List connections to/from each port\n");
 	fprintf (stderr, "        -l, --latency         Display per-port latency in frames at each port\n");
 	fprintf (stderr, "        -L, --latency         Display total latency in frames at each port\n");
@@ -32,7 +34,7 @@ show_usage (void)
 	fprintf (stderr, "        -t, --type            Display port type\n");
 	fprintf (stderr, "        -h, --help            Display this help message\n");
 	fprintf (stderr, "        --version             Output version information and exit\n\n");
-	fprintf (stderr, "For more information see http://jackit.sourceforge.net/\n");
+	fprintf (stderr, "For more information see http://jackaudio.org/\n");
 }
 
 int
@@ -41,7 +43,9 @@ main (int argc, char *argv[])
 	jack_client_t *client;
 	jack_status_t status;
 	const char **ports, **connections;
-	unsigned int i, j;
+	unsigned int i, j, k;
+	int skip_port;
+	int show_aliases = 0;
 	int show_con = 0;
 	int show_port_latency = 0;
 	int show_total_latency = 0;
@@ -49,8 +53,10 @@ main (int argc, char *argv[])
 	int show_type = 0;
 	int c;
 	int option_index;
-
+	char* aliases[2];
+	
 	struct option long_options[] = {
+		{ "aliases", 0, 0, 'A' },
 		{ "connections", 0, 0, 'c' },
 		{ "port-latency", 0, 0, 'l' },
 		{ "total-latency", 0, 0, 'L' },
@@ -68,8 +74,13 @@ main (int argc, char *argv[])
 		my_name ++;
 	}
 
-	while ((c = getopt_long (argc, argv, "clLphvt", long_options, &option_index)) >= 0) {
+	while ((c = getopt_long (argc, argv, "AclLphvt", long_options, &option_index)) >= 0) {
 		switch (c) {
+		case 'A':
+			aliases[0] = (char *) malloc (jack_port_name_size());
+			aliases[1] = (char *) malloc (jack_port_name_size());
+			show_aliases = 1;
+			break;
 		case 'c':
 			show_con = 1;
 			break;
@@ -119,10 +130,29 @@ main (int argc, char *argv[])
 	ports = jack_get_ports (client, NULL, NULL, 0);
 
 	for (i = 0; ports[i]; ++i) {
+		// skip over any that don't match ALL of the strings presented at command line
+		skip_port = 0;
+		for(k=optind; k < argc; k++){
+			if(strstr(ports[i], argv[k]) == NULL ){
+				skip_port = 1;
+			}
+		}
+		if(skip_port) continue;
+
 		printf ("%s\n", ports[i]);
 
 		jack_port_t *port = jack_port_by_name (client, ports[i]);
 
+		if (show_aliases) {
+			int cnt;
+			int i;
+
+			cnt = jack_port_get_aliases (port, aliases);
+			for (i = 0; i < cnt; ++i) {
+				printf ("   %s\n", aliases[i]);
+			}
+		}
+				
 		if (show_con) {
 			if ((connections = jack_port_get_all_connections (client, jack_port_by_name(client, ports[i]))) != 0) {
 				for (j = 0; connections[j]; j++) {
