@@ -21,7 +21,6 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: internal.h 948 2006-05-15 17:12:44Z pbd $
 */
 
 #ifndef __jack_internal_h__
@@ -171,13 +170,16 @@ typedef enum  {
   PortUnregistered,
   XRun,
   StartFreewheel,
-  StopFreewheel
+  StopFreewheel,
+  ClientRegistered,
+  ClientUnregistered
 } JackEventType;
 
 typedef struct {
     JackEventType type;
     union {
 	uint32_t n;
+        char name[JACK_CLIENT_NAME_SIZE];    
 	jack_port_id_t port_id;
 	jack_port_id_t self_id;
     } x;
@@ -223,6 +225,7 @@ typedef volatile struct {
     volatile uint64_t	signalled_at;
     volatile uint64_t	awake_at;
     volatile uint64_t	finished_at;
+    volatile int32_t	last_status;         /* w: client, r: engine and client */
 
     /* JOQ: all these pointers are trouble for 32/64 compatibility,
      * they should move to non-shared memory. 
@@ -240,6 +243,8 @@ typedef volatile struct {
     void *srate_arg;
     JackPortRegistrationCallback port_register;
     void *port_register_arg;
+    JackPortConnectCallback port_connect;
+    void *port_connect_arg;
     JackGraphOrderCallback graph_order;
     void *graph_order_arg;
     JackXRunCallback xrun;
@@ -250,7 +255,9 @@ typedef volatile struct {
     void *timebase_arg;
     JackFreewheelCallback freewheel_cb;
     void *freewheel_arg;
-	
+    JackClientRegistrationCallback client_register;	
+    void *client_register_arg;
+
     /* external clients: set by libjack
      * internal clients: set by engine */
     int (*deliver_request)(void*, jack_request_t*); /* JOQ: 64/32 bug! */
@@ -330,7 +337,8 @@ typedef enum {
 	IntClientLoad = 20,
 	IntClientName = 21,
 	IntClientUnload = 22,
-	RecomputeTotalLatencies = 23
+	RecomputeTotalLatencies = 23,
+	RecomputeTotalLatency = 24
 } RequestType;
 
 struct _jack_request {
@@ -456,6 +464,7 @@ jack_port_t *jack_port_by_id_int (const jack_client_t *client,
 
 jack_port_t *jack_port_by_name_int (jack_client_t *client,
 				    const char *port_name);
+int jack_port_name_equals (jack_port_shared_t* port, const char* target);
 
 #ifdef __GNUC__
 #  define likely(x)	__builtin_expect((x),1)
