@@ -16,7 +16,6 @@
     along with this program; if not, write to the Free Software 
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: jack.h 895 2005-05-11 02:06:47Z pbd $
 */
 
 #ifndef __jack_h__
@@ -73,25 +72,9 @@ jack_client_t *jack_client_open (const char *client_name,
 				 jack_status_t *status, ...);
 
 /**
- * Attempt to become an external client of the Jack server.
+ * \bold THIS FUNCTION IS DEPRECATED AND SHOULD NOT BE USED IN
+ *  NEW JACK CLIENTS
  *
- * JACK is evolving a mechanism for automatically starting the server
- * when needed.  As a transition, jack_client_new() only does this
- * when \$JACK_START_SERVER is defined in the environment of the
- * calling process.  In the future this will become normal behavior.
- * For full control of this feature, use jack_client_open(), instead.
- * In either case, defining \$JACK_NO_START_SERVER disables this
- * feature.
- *
- * @param client_name of at most jack_client_name_size() characters.
- * If this name is already in use, the request fails.
- *
- * @return Opaque client handle if successful, otherwise NULL.
- *
- * @note Failure generally means that the JACK server is not running.
- * If there was some other problem, it will be reported via the @ref
- * jack_error_callback mechanism.  Use jack_client_open() and check
- * the @a status parameter for more detailed information.
  */
 jack_client_t *jack_client_new (const char *client_name);
 
@@ -196,6 +179,18 @@ void jack_on_shutdown (jack_client_t *client,
 int jack_set_process_callback (jack_client_t *client,
 			       JackProcessCallback process_callback,
 			       void *arg);
+/**
+ * Block until this JACK client should process data.
+ * 
+ * The @a status argument typically indicates the result
+ * of some recent data processing.
+ * 
+ * @param client - pointer to a JACK client structure
+ * @param status - if non-zero, calling thread should exit
+ *
+ * @return the number of frames of data to process
+ */
+jack_nframes_t jack_thread_wait (jack_client_t*, int status);
 
 /**
  * Tell JACK to call @a thread_init_callback once just after
@@ -294,6 +289,16 @@ int jack_set_sample_rate_callback (jack_client_t *client,
 				   JackSampleRateCallback srate_callback,
 				   void *arg);
 
+/**
+ * Tell the JACK server to call @a registration_callback whenever a
+ * port is registered or unregistered, passing @a arg as a parameter.
+ *
+ * @return 0 on success, otherwise a non-zero error code
+ */
+int jack_set_client_registration_callback (jack_client_t *,
+					   JackClientRegistrationCallback
+					   registration_callback, void *arg);
+	
 /**
  * Tell the JACK server to call @a registration_callback whenever a
  * port is registered or unregistered, passing @a arg as a parameter.
@@ -304,6 +309,15 @@ int jack_set_port_registration_callback (jack_client_t *,
 					 JackPortRegistrationCallback
 					 registration_callback, void *arg);
 
+/**
+ * Tell the JACK server to call @a connect_callback whenever a
+ * port is connected or disconnected, passing @a arg as a parameter.
+ *
+ * @return 0 on success, otherwise a non-zero error code
+ */
+int jack_set_port_connect_callback (jack_client_t *,
+				    JackPortConnectCallback
+				    connect_callback, void *arg);
 /**
  * Tell the JACK server to call @a graph_callback whenever the
  * processing graph is reordered, passing @a arg as a parameter.
@@ -508,22 +522,6 @@ int  jack_port_tie (jack_port_t *src, jack_port_t *dst);
  */
 int  jack_port_untie (jack_port_t *port);
 
-/**
- * A client may call this function to prevent other objects
- * from changing the connection status of a port. The port
- * must be owned by the calling client.
- *
- * @return 0 on success, otherwise a non-zero error code
- */
-int jack_port_lock (jack_client_t *, jack_port_t *);
-
-/**
- * This allows other objects to change the connection status of a port.
- *
- * @return 0 on success, otherwise a non-zero error code
- */
-int jack_port_unlock (jack_client_t *, jack_port_t *);
-
 /** 
  * @return the time (in frames) between data being available or
  * delivered at/to a port, and the time at which it arrived at or is
@@ -557,7 +555,30 @@ jack_nframes_t jack_port_get_total_latency (jack_client_t *,
 void jack_port_set_latency (jack_port_t *, jack_nframes_t);
 	
 /**
- *
+ * Request a complete recomputation of a port's total latency. This
+ * can be called by a client that has just changed the internal
+ * latency of its port using @function jack_port_set_latency
+ * and wants to ensure that all signal pathways in the graph
+ * are updated with respect to the values that will be returned
+ * by @function jack_port_get_total_latency. 
+ * 
+ * @return zero for successful execution of the request. non-zero
+ *         otherwise.
+ */
+int jack_recompute_total_latency (jack_client_t*, jack_port_t* port);
+
+/**
+ * Request a complete recomputation of all port latencies. This
+ * can be called by a client that has just changed the internal
+ * latency of its port using @function jack_port_set_latency
+ * and wants to ensure that all signal pathways in the graph
+ * are updated with respect to the values that will be returned
+ * by @function jack_port_get_total_latency. It allows a client 
+ * to change multiple port latencies without triggering a 
+ * recompute for each change.
+ * 
+ * @return zero for successful execution of the request. non-zero
+ *         otherwise.
  */
 int jack_recompute_total_latencies (jack_client_t*);
 
@@ -570,6 +591,38 @@ int jack_recompute_total_latencies (jack_client_t*);
  */
 int jack_port_set_name (jack_port_t *port, const char *port_name);
 
+/**
+ * Set @a alias as an alias for @a port.  May be called at any time.
+ * If the alias is longer than jack_port_name_size(), it will be truncated.
+ * 
+ * After a successful call, and until JACK exits or
+ * @function jack_port_unset_alias() is called, @alias may be
+ * used as a alternate name for the port.
+ *
+ * Ports can have up to two aliases - if both are already 
+ * set, this function will return an error.
+ *
+ * @return 0 on success, otherwise a non-zero error code.
+ */
+int jack_port_set_alias (jack_port_t *port, const char *alias);
+
+/**
+ * Remove @a alias as an alias for @a port.  May be called at any time.
+ * 
+ * After a successful call, @a alias can no longer be 
+ * used as a alternate name for the port.
+ *
+ * @return 0 on success, otherwise a non-zero error code.
+ */
+int jack_port_unset_alias (jack_port_t *port, const char *alias);
+
+/*
+ * Get any aliases known for @port.
+ *
+ * @return the number of aliases discovered for the port
+ */
+int jack_port_get_aliases (const jack_port_t *port, char* const aliases[2]);
+
 /**
  * If @ref JackPortCanMonitor is set for this @a port, turn input
  * monitoring on or off.  Otherwise, do nothing.
@@ -751,6 +804,26 @@ jack_nframes_t jack_frame_time (const jack_client_t *);
  */
 jack_nframes_t jack_last_frame_time (const jack_client_t *client);
 
+/**
+ * @return estimated time in microseconds of the specified frame time
+ */
+jack_time_t jack_frames_to_time(const jack_client_t *client, jack_nframes_t);
+
+/**
+ * @return estimated time in frames for the specified system time.
+ */
+jack_nframes_t jack_time_to_frames(const jack_client_t *client, jack_time_t);
+
+/**
+ * @return return JACK's current system time in microseconds,
+ *         using JACK clock source. 
+ * 
+ * The value returned is guaranteed to be monotonic, but not linear.
+ *
+ * This function is a client version of @function jack_get_microseconds().
+ */
+jack_time_t jack_get_time();
+
 /**
  * @return the current CPU load estimated by JACK.  This is a running
  * average of the time it takes to execute a full process cycle for
