@@ -19,7 +19,6 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *  $Id: clientengine.c 985 2006-08-29 15:21:17Z paul $
  */
 
 #include <config.h>
@@ -96,7 +95,7 @@ jack_zombify_client (jack_engine_t *engine, jack_client_internal_t *client)
 	/* this stops jack_deliver_event() from doing anything */
 
 	client->control->dead = TRUE;
-	
+
 	jack_client_disconnect_ports (engine, client);
 	jack_client_do_deactivate (engine, client, FALSE);
 }
@@ -202,10 +201,12 @@ jack_remove_clients (jack_engine_t* engine)
 					 client->control->name,
 					 jack_client_state_name (client),
 					 client->error);
-				jack_zombify_client (engine,
-						     (jack_client_internal_t *)
-						     node->data);
-				client->error = 0;
+				if (!engine->nozombies) {
+					jack_zombify_client (engine,
+							     (jack_client_internal_t *)
+							     node->data);
+					client->error = 0;
+				}
 			}
 			
 			need_sort = TRUE;
@@ -474,8 +475,12 @@ jack_setup_client_control (jack_engine_t *engine, int fd,
 	client->control->xrun_arg = NULL;
 	client->control->port_register = NULL;
 	client->control->port_register_arg = NULL;
+	client->control->port_connect = NULL;
+	client->control->port_connect_arg = NULL;
 	client->control->graph_order = NULL;
 	client->control->graph_order_arg = NULL;
+	client->control->client_register = NULL;
+	client->control->client_register_arg = NULL;
 
 	jack_transport_client_new (client);
         
@@ -717,6 +722,8 @@ jack_client_create (jack_engine_t *engine, int client_fd)
 		close (client_fd);
 	}
 
+	jack_client_registration_notify (engine, (const char*) client->control->name, 1);
+
 	return 0;
 }
 
@@ -856,6 +863,8 @@ jack_client_disconnect (jack_engine_t *engine, int fd)
 void
 jack_client_delete (jack_engine_t *engine, jack_client_internal_t *client)
 {
+	jack_client_registration_notify (engine, (const char*) client->control->name, 0);
+
 	if (jack_client_is_internal (client)) {
 
 		jack_client_unload (client);
