@@ -18,7 +18,6 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *  $Id: clientengine.h 863 2005-01-02 15:22:48Z joq $
  */
 
 static inline int 
@@ -58,3 +57,5 @@ void	jack_intclient_name_request (jack_engine_t *engine,
 void	jack_intclient_unload_request (jack_engine_t *engine,
 				       jack_request_t *req);
 void	jack_remove_clients (jack_engine_t* engine);
+void    jack_client_registration_notify (jack_engine_t *engine,
+					 const char* name, int yn);
