@@ -17,7 +17,6 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: engine.c 1011 2007-02-23 23:49:09Z paul $
 */
 
 #include <config.h>
@@ -95,7 +94,7 @@ static int  jack_port_do_disconnect (jack_engine_t *engine,
 static int  jack_port_do_disconnect_all (jack_engine_t *engine,
 					 jack_port_id_t);
 static int  jack_port_do_unregister (jack_engine_t *engine, jack_request_t *);
-static int  jack_port_do_register (jack_engine_t *engine, jack_request_t *);
+static int  jack_port_do_register (jack_engine_t *engine, jack_request_t *, int);
 static int  jack_do_get_port_connections (jack_engine_t *engine,
 					  jack_request_t *req, int reply_fd);
 static int  jack_port_disconnect_internal (jack_engine_t *engine,
@@ -109,6 +108,10 @@ static int  jack_deliver_event (jack_engine_t *, jack_client_internal_t *,
 				jack_event_t *);
 static void jack_deliver_event_to_all (jack_engine_t *engine,
 				       jack_event_t *event);
+static void jack_notify_all_port_interested_clients (jack_engine_t *engine,
+						     jack_port_id_t a,
+						     jack_port_id_t b,
+						     int connect);
 static void jack_engine_post_process (jack_engine_t *);
 static int  jack_use_driver (jack_engine_t *engine, jack_driver_t *driver);
 static int  jack_run_cycle (jack_engine_t *engine, jack_nframes_t nframes,
@@ -124,6 +127,7 @@ static int jack_client_sort (jack_client_internal_t *a,
 			     jack_client_internal_t *b);
 static void jack_check_acyclic (jack_engine_t* engine);
 static void jack_compute_all_port_total_latencies (jack_engine_t *engine);
+static void jack_compute_port_total_latency (jack_engine_t *engine, jack_port_shared_t*);
 
 
 static inline int 
@@ -174,7 +178,15 @@ make_directory (const char *path)
 	if (stat (path, &statbuf)) {
 
 		if (errno == ENOENT) {
-			if (mkdir (path, 0700) < 0){
+			int mode;
+
+			if (getenv ("JACK_PROMISCUOUS_SERVER")) {
+				mode = 0777;
+			} else {
+				mode = 0700;
+			}
+
+			if (mkdir (path, mode) < 0){
 				jack_error ("cannot create %s directory (%s)\n",
 					    path, strerror (errno));
 				return -1;
@@ -325,7 +337,8 @@ jack_engine_place_port_buffers (jack_engine_t* engine,
 				jack_port_type_id_t ptid,
 				jack_shmsize_t one_buffer,
 				jack_shmsize_t size,
-				unsigned long nports)
+				unsigned long nports,
+				jack_nframes_t nframes)
 {
 	jack_shmsize_t offset;		/* shared memory offset */
 	jack_port_buffer_info_t *bi;
@@ -401,7 +414,7 @@ jack_engine_place_port_buffers (jack_engine_t* engine,
 
 		bi = pti->info;
 		for (i=0; i<nports; ++i, ++bi)
-			pfuncs->buffer_init(shm_segment + bi->offset, one_buffer);
+			pfuncs->buffer_init(shm_segment + bi->offset, one_buffer, nframes);
 	}
 
 	pthread_mutex_unlock (&pti->lock);
@@ -459,7 +472,7 @@ jack_resize_port_segment (jack_engine_t *engine,
 		}
 	}
 
-	jack_engine_place_port_buffers (engine, ptid, one_buffer, size, nports);
+	jack_engine_place_port_buffers (engine, ptid, one_buffer, size, nports, engine->control->buffer_size);
 
 #ifdef USE_MLOCK
 	if (engine->control->real_time) {
@@ -1172,7 +1185,7 @@ do_request (jack_engine_t *engine, jack_request_t *req, int *reply_fd)
 
 	switch (req->type) {
 	case RegisterPort:
-		req->status = jack_port_do_register (engine, req);
+		req->status = jack_port_do_register (engine, req, reply_fd ? FALSE : TRUE);
 		break;
 
 	case UnRegisterPort:
@@ -1285,6 +1298,13 @@ do_request (jack_engine_t *engine, jack_request_t *req, int *reply_fd)
 		req->status = 0;
 		break;
 
+	case RecomputeTotalLatency:
+		jack_lock_graph (engine);
+		jack_compute_port_total_latency (engine, &engine->control->ports[req->x.port_info.port_id]);
+		jack_unlock_graph (engine);
+		req->status = 0;
+		break;
+
 	default:
 		/* some requests are handled entirely on the client
 		 * side, by adjusting the shared memory area(s) */
@@ -1529,7 +1549,7 @@ jack_engine_t *
 jack_engine_new (int realtime, int rtpriority, int do_mlock, int do_unlock,
 		 const char *server_name, int temporary, int verbose,
 		 int client_timeout, unsigned int port_max, pid_t wait_pid,
-		 jack_nframes_t frame_time_offset, JSList *drivers)
+		 jack_nframes_t frame_time_offset, int nozombies, JSList *drivers)
 {
 	jack_engine_t *engine;
 	unsigned int i;
@@ -1594,6 +1614,12 @@ jack_engine_new (int realtime, int rtpriority, int do_mlock, int do_unlock,
 	engine->freewheeling = 0;
 	engine->feedbackcount = 0;
 	engine->wait_pid = wait_pid;
+	engine->nozombies = nozombies;
+
+	engine->audio_out_cnt = 0;
+	engine->audio_in_cnt = 0;
+	engine->midi_out_cnt = 0;
+	engine->midi_in_cnt = 0;
 
 	jack_engine_reset_rolling_usecs (engine);
 	engine->max_usecs = 0.0f;
@@ -1671,6 +1697,8 @@ jack_engine_new (int realtime, int rtpriority, int do_mlock, int do_unlock,
 	for (i = 0; i < engine->port_max; i++) {
 		engine->control->ports[i].in_use = 0;
 		engine->control->ports[i].id = i;
+		engine->control->ports[i].alias1[0] = '\0';
+		engine->control->ports[i].alias2[0] = '\0';
 	}
 
 	/* allocate internal port structures so that we can keep track
@@ -2004,6 +2032,12 @@ jack_run_one_cycle (jack_engine_t *engine, jack_nframes_t nframes,
 						client->control->name);
 					client->error++;
 				}
+				if(client->control->last_status != 0) {
+					VERBOSE(engine,
+						"client %s has nonzero process callback status (%d)\n",
+						client->control->name, client->control->last_status);
+					client->error++;
+				}
 			}
 			
 			DEBUG ("client %s errors = %d", client->control->name,
@@ -2219,6 +2253,34 @@ jack_deliver_event_to_all (jack_engine_t *engine, jack_event_t *event)
 	jack_unlock_graph (engine);
 }
 
+static void
+jack_notify_all_port_interested_clients (jack_engine_t *engine, jack_port_id_t a, jack_port_id_t b, int connected)
+{
+	JSList *node;
+	jack_event_t event;
+
+	event.type = (connected ? PortConnected : PortDisconnected);
+	event.x.self_id = a;
+	event.y.other_id = b;
+	
+	/* GRAPH MUST BE LOCKED : see callers of jack_send_connection_notification() 
+	 */
+
+	for (node = engine->clients; node; node = jack_slist_next (node)) {
+		jack_client_internal_t* client;
+
+		client = (jack_client_internal_t*) node->data;
+
+		if (client->control->port_connect != NULL) {
+			
+			/* one of the ports belong to this client or it has a port connect callback */
+			
+			jack_deliver_event (engine, client, &event);
+			
+		} 
+	}
+}
+
 static int
 jack_deliver_event (jack_engine_t *engine, jack_client_internal_t *client,
 		    jack_event_t *event)
@@ -2602,13 +2664,24 @@ jack_get_port_total_latency (jack_engine_t *engine,
 	return latency + max_latency;
 }
 
+static void
+jack_compute_port_total_latency (jack_engine_t* engine, jack_port_shared_t* port)
+{
+	if (port->in_use) {
+		port->total_latency =
+			jack_get_port_total_latency (
+				engine, &engine->internal_ports[port->id],
+				0, !(port->flags & JackPortIsOutput));
+	}
+}
+
 static void
 jack_compute_all_port_total_latencies (jack_engine_t *engine)
 {
 	jack_port_shared_t *shared = engine->control->ports;
 	unsigned int i;
-	int toward_port;
-
+ 	int toward_port;
+ 
 	for (i = 0; i < engine->control->port_max; i++) {
 		if (shared[i].in_use) {
 			if (shared[i].flags & JackPortIsOutput) {
@@ -2914,18 +2987,6 @@ jack_port_do_connect (jack_engine_t *engine,
 		return -1;
 	}
 
-	if (srcport->shared->locked) {
-		jack_error ("source port %s is locked against connection"
-			    " changes", source_port);
-		return -1;
-	}
-
-	if (dstport->shared->locked) {
-		jack_error ("destination port %s is locked against connection"
-			    " changes", destination_port);
-		return -1;
-	}
-
 	if (srcport->shared->ptype_id != dstport->shared->ptype_id) {
 		jack_error ("ports used in attemped connection are not of "
 			    "the same data type");
@@ -3072,10 +3133,16 @@ jack_port_do_connect (jack_engine_t *engine,
 		jack_send_connection_notification (engine,
 						   srcport->shared->client_id,
 						   src_id, dst_id, TRUE);
+		
+
 		jack_send_connection_notification (engine,
 						   dstport->shared->client_id,
 						   dst_id, src_id, TRUE);
 						   
+		/* send a port connection notification just once to everyone who cares */
+
+		jack_notify_all_port_interested_clients (engine, src_id, dst_id, 1);
+
 		jack_sort_graph (engine);
 	}
 
@@ -3137,6 +3204,10 @@ jack_port_disconnect_internal (jack_engine_t *engine,
 				engine, dstport->shared->client_id, dst_id,
 				src_id, FALSE);
 
+			/* send a port connection notification just once to everyone who cares */
+			
+			jack_notify_all_port_interested_clients (engine, src_id, dst_id, 0);
+
 			if (connect->dir) {
 			
 				jack_client_internal_t *src;
@@ -3329,6 +3400,8 @@ jack_use_driver (jack_engine_t *engine, jack_driver_t *driver)
 		engine->driver = 0;
 	}
 
+	engine->driver = driver;
+
 	if (driver) {
 		if (driver->attach (driver, engine))
 			return -1;
@@ -3337,7 +3410,6 @@ jack_use_driver (jack_engine_t *engine, jack_driver_t *driver)
 			jack_rolling_interval (driver->period_usecs);
 	}
 
-	engine->driver = driver;
 	return 0;
 }
 
@@ -3374,6 +3446,8 @@ jack_port_release (jack_engine_t *engine, jack_port_internal_t *port)
 {
 	pthread_mutex_lock (&engine->port_lock);
 	port->shared->in_use = 0;
+	port->shared->alias1[0] = '\0';
+	port->shared->alias2[0] = '\0';
 
 	if (port->buffer_info) {
 		jack_port_buffer_list_t *blist =
@@ -3396,7 +3470,7 @@ jack_get_port_internal_by_name (jack_engine_t *engine, const char *name)
 	pthread_mutex_lock (&engine->port_lock);
 
 	for (id = 0; id < engine->port_max; id++) {
-		if (strcmp (engine->control->ports[id].name, name) == 0) {
+		if (jack_port_name_equals (&engine->control->ports[id], name)) {
 			break;
 		}
 	}
@@ -3411,14 +3485,15 @@ jack_get_port_internal_by_name (jack_engine_t *engine, const char *name)
 }
 
 int
-jack_port_do_register (jack_engine_t *engine, jack_request_t *req)
-	
+jack_port_do_register (jack_engine_t *engine, jack_request_t *req, int internal)
 {
 	jack_port_id_t port_id;
 	jack_port_shared_t *shared;
 	jack_port_internal_t *port;
 	jack_client_internal_t *client;
 	unsigned long i;
+	char *backend_client_name;
+	size_t len;
 
 	for (i = 0; i < engine->control->n_port_types; ++i) {
 		if (strcmp (req->x.port_info.type,
@@ -3438,24 +3513,69 @@ jack_port_do_register (jack_engine_t *engine, jack_request_t *req)
 						  req->x.port_info.client_id))
 	    == NULL) {
 		jack_error ("unknown client id in port registration request");
+		jack_unlock_graph (engine);
+		return -1;
+	}
+
+	if ((port = jack_get_port_by_name(engine, req->x.port_info.name)) != NULL) {
+		jack_error ("duplicate port name in port registration request");
+		jack_unlock_graph (engine);
 		return -1;
 	}
-	jack_unlock_graph (engine);
 
 	if ((port_id = jack_get_free_port (engine)) == (jack_port_id_t) -1) {
 		jack_error ("no ports available!");
+		jack_unlock_graph (engine);
 		return -1;
 	}
 
 	shared = &engine->control->ports[port_id];
 
+	if (!internal || !engine->driver)
+		goto fallback;
+
+	backend_client_name = (char *) engine->driver->internal_client->control->name;
+	len = strlen (backend_client_name);
+
+	if (strncmp (req->x.port_info.name, backend_client_name, len) != 0)
+		goto fallback;
+
+	/* use backend's original as an alias, use predefined names */
+
+	if (strcmp(req->x.port_info.type, JACK_DEFAULT_AUDIO_TYPE) == 0) {
+		if ((req->x.port_info.flags & (JackPortIsPhysical|JackPortIsInput)) == (JackPortIsPhysical|JackPortIsInput)) {
+			snprintf (shared->name, sizeof (shared->name), JACK_BACKEND_ALIAS ":playback_%d", ++engine->audio_out_cnt);
+			strcpy (shared->alias1, req->x.port_info.name);
+			goto next;
+		} 
+		else if ((req->x.port_info.flags & (JackPortIsPhysical|JackPortIsOutput)) == (JackPortIsPhysical|JackPortIsOutput)) {
+			snprintf (shared->name, sizeof (shared->name), JACK_BACKEND_ALIAS ":capture_%d", ++engine->audio_in_cnt);
+			strcpy (shared->alias1, req->x.port_info.name);
+			goto next;
+		}
+	}
+	else if (strcmp(req->x.port_info.type, JACK_DEFAULT_MIDI_TYPE) == 0) {
+		if ((req->x.port_info.flags & (JackPortIsPhysical|JackPortIsInput)) == (JackPortIsPhysical|JackPortIsInput)) {
+			snprintf (shared->name, sizeof (shared->name), JACK_BACKEND_ALIAS ":midi_playback_%d", ++engine->midi_out_cnt);
+			strcpy (shared->alias1, req->x.port_info.name);
+			goto next;
+		} 
+		else if ((req->x.port_info.flags & (JackPortIsPhysical|JackPortIsOutput)) == (JackPortIsPhysical|JackPortIsOutput)) {
+			snprintf (shared->name, sizeof (shared->name), JACK_BACKEND_ALIAS ":midi_capture_%d", ++engine->midi_in_cnt);
+			strcpy (shared->alias1, req->x.port_info.name);
+			goto next;
+		}
+	}
+
+fallback:
 	strcpy (shared->name, req->x.port_info.name);
+
+next:
 	shared->ptype_id = engine->control->port_types[i].ptype_id;
 	shared->client_id = req->x.port_info.client_id;
 	shared->flags = req->x.port_info.flags;
 	shared->latency = 0;
 	shared->monitor_requests = 0;
-	shared->locked = 0;
 
 	port = &engine->internal_ports[port_id];
 
@@ -3465,10 +3585,11 @@ jack_port_do_register (jack_engine_t *engine, jack_request_t *req)
 	
 	if (jack_port_assign_buffer (engine, port)) {
 		jack_error ("cannot assign buffer for port");
+		jack_port_release (engine, &engine->internal_ports[port_id]);
+		jack_unlock_graph (engine);
 		return -1;
 	}
 
-	jack_lock_graph (engine);
 	client->ports = jack_slist_prepend (client->ports, port);
 	jack_port_registration_notify (engine, port_id, TRUE);
 	jack_unlock_graph (engine);
@@ -3652,6 +3773,41 @@ jack_port_registration_notify (jack_engine_t *engine,
 	}
 }
 
+void
+jack_client_registration_notify (jack_engine_t *engine,
+				 const char* name, int yn)
+{
+	jack_event_t event;
+	jack_client_internal_t *client;
+	JSList *node;
+
+	event.type = (yn ? ClientRegistered : ClientUnregistered);
+	snprintf (event.x.name, sizeof (event.x.name), "%s", name);
+	
+	for (node = engine->clients; node; node = jack_slist_next (node)) {
+		
+		client = (jack_client_internal_t *) node->data;
+
+		if (!client->control->active) {
+			continue;
+		}
+
+		if (strcmp ((char*) client->control->name, (char*) name) == 0) {
+			/* do not notify client of its own registration */
+			continue;
+		}
+
+		if (client->control->client_register) {
+			if (jack_deliver_event (engine, client, &event)) {
+				jack_error ("cannot send client registration"
+					    " notification to %s (%s)",
+					     client->control->name,
+					    strerror (errno));
+			}
+		}
+	}
+}
+
 int
 jack_port_assign_buffer (jack_engine_t *engine, jack_port_internal_t *port)
 {
@@ -3696,7 +3852,7 @@ jack_get_port_by_name (jack_engine_t *engine, const char *name)
 
 	for (id = 0; id < engine->port_max; id++) {
 		if (engine->control->ports[id].in_use &&
-		    strcmp (engine->control->ports[id].name, name) == 0) {
+		    jack_port_name_equals (&engine->control->ports[id], name)) {
 			return &engine->internal_ports[id];
 		}
 	}
@@ -3712,8 +3868,8 @@ jack_send_connection_notification (jack_engine_t *engine,
 
 {
 	jack_client_internal_t *client;
-	jack_event_t event;
-
+ 	jack_event_t event;
+ 
 	if ((client = jack_client_internal_by_id (engine, client_id)) == NULL) {
 		jack_error ("no such client %" PRIu32
 			    " during connection notification", client_id);
