@@ -17,7 +17,6 @@
     along with this program; if not, write to the Free Software 
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: client.c 1010 2007-02-23 19:23:07Z paul $
 */
 
 #include <config.h>
@@ -62,15 +61,8 @@
 #include <sysdeps/pThreadUtilities.h>
 #endif
 
-#ifdef WITH_TIMESTAMPS
-#include <jack/timestamps.h>
-#endif /* WITH_TIMESTAMPS */
-
 static pthread_mutex_t client_lock;
 static pthread_cond_t  client_ready;
-#ifdef ARCH_X86
-int cpu_type = 0;
-#endif /* ARCH_X86 */
 
 #define EVENT_POLL_INDEX 0
 #define WAIT_POLL_INDEX 1
@@ -87,98 +79,7 @@ typedef struct {
 
 #ifdef ARCH_X86
 
-static int
-have_3dnow ()
-{
-	unsigned int res = 0;
-
-#ifdef __x86_64__
-	asm volatile ("pushq %%rbx\n\t" : : : "memory");
-#else
-	asm volatile ("pushl %%ebx\n\t" : : : "memory");
-#endif
-	asm volatile (
-		"movl $0x80000000, %%eax\n\t" \
-		"cpuid\n\t" \
-		"cmpl $0x80000001, %%eax\n\t" \
-		"jl tdnow_prexit\n\t" \
-		\
-		"movl $0x80000001, %%eax\n\t" \
-		"cpuid\n\t" \
-		\
-		"xorl %%eax, %%eax\n\t" \
-		\
-		"movl $1, %%ecx\n\t" \
-		"shll $31, %%ecx\n\t" \
-		"testl %%ecx, %%edx\n\t" \
-		"jz tdnow_testexit\n\t" \
-		"movl $1, %%eax\n\t" \
-		\
-		"movl $1, %%ecx\n\t" \
-		"shll $30, %%ecx\n\t" \
-		"testl %%ecx, %%edx\n\t" \
-		"jz tdnow_testexit\n\t" \
-		"movl $2, %%eax\n\t" \
-		"jmp tdnow_testexit\n\t" \
-		\
-		"tdnow_prexit:\n\t" \
-		"xorl %%eax, %%eax\n\t" \
-		"tdnow_testexit:\n\t"
-		: "=a" (res)
-		:
-		: "ecx", "edx", "memory");
-#ifdef __x86_64__
-	asm volatile ("popq %%rbx\n\t" : : : "memory");
-#else
-	asm volatile ("popl %%ebx\n\t" : : : "memory");
-#endif
-	return res;
-}
-
-static int
-have_sse ()
-{
-	unsigned int res = 0;
-
-#ifdef __x86_64__
-	asm volatile ("pushq %%rbx\n\t" : : : "memory");
-#else
-	asm volatile ("pushl %%ebx\n\t" : : : "memory");
-#endif
-	asm volatile (
-		"movl $1, %%eax\n\t" \
-		"cpuid\n\t" \
-		\
-		"xorl %%eax, %%eax\n\t" \
-		\
-		"movl $1, %%ebx\n\t" \
-		"shll $25, %%ebx\n\t" \
-		"testl %%ebx, %%edx\n\t" \
-		"jz sse_testexit\n\t" \
-		"movl $1, %%eax\n\t" \
-		\
-		"movl $1, %%ebx\n\t" \
-		"shll $26, %%ebx\n\t" \
-		"testl %%ebx, %%edx\n\t" \
-		"jz sse_testexit\n\t" \
-		"movl $2, %%eax\n\t" \
-		\
-		"movl $1, %%ebx\n\t" \
-		"testl %%ebx, %%ecx\n\t" \
-		"jz sse_testexit\n\t" \
-		"movl $3, %%eax\n\t" \
-		\
-		"sse_testexit:\n\t"
-		: "=a" (res)
-		:
-		: "ecx", "edx", "memory");
-#ifdef __x86_64__
-	asm volatile ("popq %%rbx\n\t" : : : "memory");
-#else
-	asm volatile ("popl %%ebx\n\t" : : : "memory");
-#endif
-	return res;
-}
+int cpu_type = 0;
 
 static void
 init_cpu ()
@@ -214,13 +115,52 @@ jack_get_tmpdir ()
 	FILE* in;
 	size_t len;
 	char buf[PATH_MAX+2]; /* allow tmpdir to live anywhere, plus newline, plus null */
+	char *pathenv;
+	char *pathcopy;
+	char *p;
 
-	if ((in = popen ("jackd -l", "r")) == NULL) {
+	/* some implementations of popen(3) close a security loophole by
+	   resetting PATH for the exec'd command. since we *want* to
+	   use the user's PATH setting to locate jackd, we have to
+	   do it ourselves.
+	*/
+
+	if ((pathenv = getenv ("PATH")) == 0) {
+		return -1;
+	}
+
+	/* don't let strtok(3) mess with the real environment variable */
+
+	pathcopy = strdup (pathenv);
+	p = strtok (pathcopy, ":");
+
+	while (p) {
+		char jackd[PATH_MAX+1];
+		char command[PATH_MAX+4];
+
+		snprintf (jackd, sizeof (jackd), "%s/jackd", p);
+		
+		if (access (jackd, X_OK) == 0) {
+			
+			snprintf (command, sizeof (command), "%s -l", jackd);
+
+			if ((in = popen (command, "r")) != NULL) {
+				break;
+			}
+		}
+
+		p = strtok (NULL, ":");
+	}
+
+	if (p == NULL) {
+		/* no command successfully started */
+		free (pathcopy);
 		return -1;
 	}
 
 	if (fgets (buf, sizeof (buf), in) == NULL) {
 		fclose (in);
+		free (pathcopy);
 		return -1;
 	}
 
@@ -229,6 +169,7 @@ jack_get_tmpdir ()
 	if (buf[len-1] != '\n') {
 		/* didn't get a whole line */
 		fclose (in);
+		free (pathcopy);
 		return -1;
 	}
 
@@ -236,9 +177,8 @@ jack_get_tmpdir ()
 	memcpy (jack_tmpdir, buf, len-1);
 	jack_tmpdir[len-1] = '\0';
 	
-	fprintf (stderr, "JACK tmpdir identified as [%s]\n", jack_tmpdir);
-
 	fclose (in);
+	free (pathcopy);
 	return 0;
 }
 
@@ -429,54 +369,68 @@ int
 jack_client_handle_port_connection (jack_client_t *client, jack_event_t *event)
 {
 	jack_port_t *control_port;
-	jack_port_t *other;
+	jack_port_t *other = 0;
 	JSList *node;
 	int need_free = FALSE;
 
-	switch (event->type) {
-	case PortConnected:
-		other = jack_port_new (client, event->y.other_id,
-				       client->engine);
-		/* jack_port_by_id_int() always returns an internal
-		 * port that does not need to be deallocated */
-		control_port = jack_port_by_id_int (client, event->x.self_id,
-						    &need_free);
-		pthread_mutex_lock (&control_port->connection_lock);
-		control_port->connections =
-			jack_slist_prepend (control_port->connections,
-					    (void *) other);
-		pthread_mutex_unlock (&control_port->connection_lock);
-		break;
+	if (client->engine->ports[event->x.self_id].client_id == client->control->id ||
+	    client->engine->ports[event->y.other_id].client_id == client->control->id) {
 
-	case PortDisconnected:
-		/* jack_port_by_id_int() always returns an internal
-		 * port that does not need to be deallocated */
-		control_port = jack_port_by_id_int (client, event->x.self_id,
-						    &need_free);
-		pthread_mutex_lock (&control_port->connection_lock);
+		/* its one of ours */
 
-		for (node = control_port->connections; node;
-		     node = jack_slist_next (node)) {
+		switch (event->type) {
+		case PortConnected:
+			other = jack_port_new (client, event->y.other_id,
+					       client->engine);
+			/* jack_port_by_id_int() always returns an internal
+			 * port that does not need to be deallocated 
+			 */
+			control_port = jack_port_by_id_int (client, event->x.self_id,
+							    &need_free);
+			pthread_mutex_lock (&control_port->connection_lock);
+			control_port->connections =
+				jack_slist_prepend (control_port->connections,
+						    (void *) other);
+			pthread_mutex_unlock (&control_port->connection_lock);
+			break;
+			
+		case PortDisconnected:
+			/* jack_port_by_id_int() always returns an internal
+			 * port that does not need to be deallocated 
+			 */
+			control_port = jack_port_by_id_int (client, event->x.self_id,
+							    &need_free);
+			pthread_mutex_lock (&control_port->connection_lock);
+			
+			for (node = control_port->connections; node;
+			     node = jack_slist_next (node)) {
+				
+				other = (jack_port_t *) node->data;
 
-			other = (jack_port_t *) node->data;
-
-			if (other->shared->id == event->y.other_id) {
-				control_port->connections =
-					jack_slist_remove_link (
-						control_port->connections,
-						node);
-				jack_slist_free_1 (node);
-				free (other);
-				break;
+				if (other->shared->id == event->y.other_id) {
+					control_port->connections =
+						jack_slist_remove_link (
+							control_port->connections,
+							node);
+					jack_slist_free_1 (node);
+					free (other);
+					break;
+				}
 			}
+			
+			pthread_mutex_unlock (&control_port->connection_lock);
+			break;
+			
+		default:
+			/* impossible */
+			break;
 		}
+	}
 
-		pthread_mutex_unlock (&control_port->connection_lock);
-		break;
-
-	default:
-		/* impossible */
-		break;
+	if (client->control->port_connect) {
+		client->control->port_connect (event->x.self_id, event->y.other_id,
+					       (event->type == PortConnected ? 1 : 0), 
+					       client->control->port_connect_arg);
 	}
 
 	return 0;
@@ -507,6 +461,8 @@ jack_handle_reorder (jack_client_t *client, jack_event_t *event)
 {	
 	char path[PATH_MAX+1];
 
+	DEBUG ("graph reorder\n");
+
 	if (client->graph_wait_fd >= 0) {
 		DEBUG ("closing graph_wait_fd==%d", client->graph_wait_fd);
 		close (client->graph_wait_fd);
@@ -654,7 +610,7 @@ _start_server (const char *server_name)
 	int i = 0;
 	int good = 0;
 	int ret;
-	
+
 	snprintf(filename, 255, "%s/.jackdrc", getenv("HOME"));
 	fp = fopen(filename, "r");
 
@@ -727,6 +683,14 @@ _start_server (const char *server_name)
 		++i;
 	}
 	argv[i] = 0;
+
+#if 0
+	fprintf (stderr, "execing  JACK using %s\n", command);
+	for (_xx = 0; argv[_xx]; ++_xx) {
+		fprintf (stderr, "\targv[%d] = %s\n", _xx, argv[_xx]);
+	}
+#endif
+
 	execv (command, argv);
 
 	/* If execv() succeeds, it does not return.  There's no point
@@ -807,7 +771,7 @@ jack_request_client (ClientType type,
 			    va->load_init, sizeof (req.object_data) - 1);
 		return -1;
 	}
-
+	
 	if ((*req_fd = server_connect (va->server_name)) < 0) {
 		int trys;
 		if (start_server(va->server_name, options)) {
@@ -826,6 +790,7 @@ jack_request_client (ClientType type,
 	}
 
 	/* format connection request */
+
 	req.protocol_v = jack_protocol_version;
 	req.load = TRUE;
 	req.type = type;
@@ -1162,8 +1127,13 @@ jack_user_dir (void)
 
 	/* format the path name on the first call */
 	if (user_dir[0] == '\0') {
-		snprintf (user_dir, sizeof (user_dir), "%s/jack-%d",
-			  jack_tmpdir, getuid ());
+		if (getenv ("JACK_PROMISCUOUS_SERVER")) {
+			snprintf (user_dir, sizeof (user_dir), "%s/jack",
+				  jack_tmpdir);
+		} else {
+			snprintf (user_dir, sizeof (user_dir), "%s/jack-%d",
+				  jack_tmpdir, getuid ());
+		}
 	}
 
 	return user_dir;
@@ -1216,6 +1186,16 @@ jack_recompute_total_latencies (jack_client_t* client)
 	return jack_client_deliver_request (client, &request);
 }
 
+int
+jack_recompute_total_latency (jack_client_t* client, jack_port_t* port)
+{
+	jack_request_t request;
+
+	request.type = RecomputeTotalLatency;
+	request.x.port_info.port_id = port->shared->id;
+	return jack_client_deliver_request (client, &request);
+}
+
 int
 jack_set_freewheel (jack_client_t* client, int onoff)
 {
@@ -1263,329 +1243,9 @@ jack_stop_freewheel (jack_client_t* client)
 	}
 }
 
-static void *
-jack_client_thread (void *arg)
+static void
+jack_client_thread_suicide (jack_client_t* client)
 {
-	jack_client_t *client = (jack_client_t *) arg;
-	jack_client_control_t *control = client->control;
-	jack_event_t event;
-	char status = 0;
-	int err = 0;
-#ifndef JACK_USE_MACH_THREADS                   
-	char c = 0;
-#endif
-
-	pthread_mutex_lock (&client_lock);
-	client->thread_ok = TRUE;
-	client->thread_id = pthread_self();
-	pthread_cond_signal (&client_ready);
-	pthread_mutex_unlock (&client_lock);
-
-	client->control->pid = getpid();
-	client->control->pgrp = getpgrp();
-
-	DEBUG ("client thread is now running");
-
-	if (client->control->thread_init) {
-		DEBUG ("calling client thread init callback");
-		client->control->thread_init (client->control->thread_init_arg);
-	}
-
-	while (err == 0) {
-	
-		if (client->engine->engine_ok == 0) {
-		     if (client->on_shutdown)
-			     client->on_shutdown (client->on_shutdown_arg);
-		     else
-			     jack_error ("engine unexpectedly shutdown; "
-					 "thread exiting\n");
-		     pthread_exit (0);
-		}
-
-		DEBUG ("client polling on %s", client->pollmax == 2 ? 
-		       "event_fd and graph_wait_fd..." :
-		       "event_fd only");
-
-		if (poll (client->pollfd, client->pollmax, 1000) < 0) {
-			if (errno == EINTR) {
-				continue;
-			}
-			jack_error ("poll failed in client (%s)",
-				    strerror (errno));
-			status = -1;
-			break;
-		}
-                
-		/* get an accurate timestamp on waking from poll for a
-		 * process() cycle. 
-		 */
-
-	#ifndef JACK_USE_MACH_THREADS 
-		if (client->graph_wait_fd >= 0
-		    && client->pollfd[WAIT_POLL_INDEX].revents & POLLIN) {
-			control->awake_at = jack_get_microseconds();
-		}
-	
-		DEBUG ("pfd[EVENT].revents = 0x%x pfd[WAIT].revents = 0x%x",
-		       client->pollfd[EVENT_POLL_INDEX].revents,
-		       client->pollfd[WAIT_POLL_INDEX].revents);
-	#endif
-	
-		pthread_testcancel();
-		
-	#ifndef JACK_USE_MACH_THREADS 
-		if (client->graph_wait_fd >= 0 &&
-		    (client->pollfd[WAIT_POLL_INDEX].revents & ~POLLIN)) {
-			
-			DEBUG ("\n\n\n\n\n\n\n\nWAITFD ERROR,"
-			       " ZOMBIE\n\n\n\n\n");
-			
-			/* our upstream "wait" connection
-			   closed, which either means that
-			   an intermediate client exited, or
-			   jackd exited, or jackd zombified
-			   us.
-
-			   we can discover the zombification
-			   via client->control->dead, but
-			   the other two possibilities are
-			   impossible to identify just from
-			   this situation. so we have to
-			   check what we are connected to,
-			   and act accordingly.
-			*/
-			
-			if (client->upstream_is_jackd) {
-                                DEBUG ("WE DIE\n");
-				goto zombie;
-			} else {
-                                DEBUG ("WE PUNT\n");
-				/* don't poll on the wait fd
-				 * again until we get a
-				 * GraphReordered event.
-				 */
-				 
-				client->graph_wait_fd = -1;
-				client->pollmax = 1;
-			}
-		}
-	#endif
-
-		if (client->control->dead) {
-			goto zombie;
-		}
-
-		if (client->pollfd[EVENT_POLL_INDEX].revents & ~POLLIN) {
-			/* jackd shutdown */
-			goto zombie;
-		}
-
-		if (client->pollfd[EVENT_POLL_INDEX].revents & POLLIN) {
-
-			DEBUG ("client receives an event, "
-			       "now reading on event fd");
-                
-			/* server has sent us an event. process the
-			 * event and reply */
-
-			if (read (client->event_fd, &event, sizeof (event))
-			    != sizeof (event)) {
-				jack_error ("cannot read server event (%s)",
-					    strerror (errno));
-				err++;
-				break;
-			}
-
-			status = 0;
-
-			switch (event.type) {
-			case PortRegistered:
-				if (control->port_register) {
-					control->port_register
-						(event.x.port_id, TRUE,
-						 control->port_register_arg);
-				} 
-				break;
-
-			case PortUnregistered:
-				if (control->port_register) {
-					control->port_register
-						(event.x.port_id, FALSE,
-						 control->port_register_arg);
-				}
-				break;
-
-			case GraphReordered:
-				status = jack_handle_reorder (client, &event);
-				break;
-
-			case PortConnected:
-			case PortDisconnected:
-				status = jack_client_handle_port_connection
-					(client, &event);
-				break;
-
-			case BufferSizeChange:
-				jack_client_invalidate_port_buffers (client);
-				if (control->bufsize) {
-					status = control->bufsize
-						(control->nframes,
-						 control->bufsize_arg);
-				} 
-				break;
-
-			case SampleRateChange:
-				if (control->srate) {
-					status = control->srate
-						(control->nframes,
-						 control->srate_arg);
-				}
-				break;
-
-			case XRun:
-				if (control->xrun) {
-					status = control->xrun
-						(control->xrun_arg);
-				}
-				break;
-
-			case AttachPortSegment:
-				jack_attach_port_segment (client, event.y.ptid);
-				break;
-				
-			case StartFreewheel:
-				jack_start_freewheel (client);
-				break;
-
-			case StopFreewheel:
-				jack_stop_freewheel (client);
-				break;
-			}
-
-			DEBUG ("client has dealt with the event, writing "
-			       "response on event fd");
-
-			if (write (client->event_fd, &status, sizeof (status))
-			    != sizeof (status)) {
-				jack_error ("cannot send event response to "
-					    "engine (%s)", strerror (errno));
-				err++;
-				break;
-			}
-		}
-		
-
-#ifndef JACK_USE_MACH_THREADS 
-		if (client->pollfd[WAIT_POLL_INDEX].revents & POLLIN) {
-
-#ifdef WITH_TIMESTAMPS
-			jack_reset_timestamps ();
-#endif
-
-			DEBUG ("client %d signalled at %" PRIu64
-			       ", awake for process at %" PRIu64
-			       " (delay = %" PRIu64
-			       " usecs) (wakeup on graph_wait_fd==%d)", 
-			       getpid(),
-			       control->signalled_at, 
-			       control->awake_at, 
-			       control->awake_at - control->signalled_at,
-			       client->pollfd[WAIT_POLL_INDEX].fd);
-
-			control->state = Running;
-
-			/* begin preemption checking */
-			CHECK_PREEMPTION (client->engine, TRUE);
-			
-			if (control->sync_cb)
-				jack_call_sync_client (client);
-
-			if (control->process) {
-				if (control->process (control->nframes,
-						      control->process_arg)
-				    == 0) {
-					control->state = Finished;
-				}
-			} else {
-				control->state = Finished;
-			}
-
-			if (control->timebase_cb)
-				jack_call_timebase_master (client);
-
-			/* end preemption checking */
-			CHECK_PREEMPTION (client->engine, FALSE);
-
-			control->finished_at = jack_get_microseconds();
-
-#ifdef WITH_TIMESTAMPS
-			jack_timestamp ("finished");
-#endif
-		
-#ifndef JACK_USE_MACH_THREADS 			
-			/* pass the execution token along */
-
-			DEBUG ("client finished processing at %" PRIu64
-			       " (elapsed = %" PRIu64
-			       " usecs), writing on graph_next_fd==%d", 
-			       control->finished_at, 
-			       control->finished_at - control->awake_at,
-			       client->graph_next_fd);
-
-			if (write (client->graph_next_fd, &c, sizeof (c))
-			    != sizeof (c)) {
-				jack_error ("cannot continue execution of the "
-					    "processing graph (%s)",
-					    strerror(errno));
-				err++;
-				break;
-			}
-
-			DEBUG ("client sent message to next stage by %" PRIu64
-			       ", client reading on graph_wait_fd==%d", 
-			       jack_get_microseconds(), client->graph_wait_fd);
-#endif
-
-#ifdef WITH_TIMESTAMPS
-			jack_timestamp ("read pending byte from wait");
-#endif
-			DEBUG("reading cleanup byte from pipe\n");
-
-#ifndef JACK_USE_MACH_THREADS
-			if ((read (client->graph_wait_fd, &c, sizeof (c))
-			     != sizeof (c))) {
-				DEBUG ("WARNING: READ FAILED!");
-#if 0
-				jack_error ("cannot complete execution of the "
-				            "processing graph (%s)",
-					    strerror(errno));
-				err++;
-				break;
-#endif
-			}
-#endif
-
-			/* check if we were killed during the process
-			 * cycle (or whatever) */
-			if (client->control->dead) {
-				goto zombie;
-			}
-
-			DEBUG("process cycle fully complete\n");
-
-#ifdef WITH_TIMESTAMPS
-			jack_timestamp ("read done");
-			jack_dump_timestamps (stdout);
-#endif			
-
-		}
-#endif
-
-	}
-	
-	return (void *) ((intptr_t)err);
-
-  zombie:
 	if (client->on_shutdown) {
 		jack_error ("zombified - calling shutdown handler");
 		client->on_shutdown (client->on_shutdown_arg);
@@ -1599,9 +1259,362 @@ jack_client_thread (void *arg)
 
 	pthread_exit (0);
 	/*NOTREACHED*/
+}
+
+static int
+jack_client_process_events (jack_client_t* client)
+{
+	jack_event_t event;
+	char status = 0;
+	jack_client_control_t *control = client->control;
+	JSList *node;
+	jack_port_t* port;
+
+	if (client->pollfd[EVENT_POLL_INDEX].revents & POLLIN) {
+		
+		DEBUG ("client receives an event, "
+		       "now reading on event fd");
+                
+		/* server has sent us an event. process the
+		 * event and reply */
+		
+		if (read (client->event_fd, &event, sizeof (event))
+		    != sizeof (event)) {
+			jack_error ("cannot read server event (%s)",
+				    strerror (errno));
+			return -1;
+		}
+		
+		status = 0;
+		
+		switch (event.type) {
+		case PortRegistered:
+			for (node = client->ports_ext; node; node = jack_slist_next (node)) {
+				port = node->data;
+				if (port->shared->id == event.x.port_id) { // Found port, update port type
+					port->type_info = &client->engine->port_types[port->shared->ptype_id];
+				}
+			}
+			if (control->port_register) {
+				control->port_register
+					(event.x.port_id, TRUE,
+					 control->port_register_arg);
+			} 
+			break;
+			
+		case PortUnregistered:
+			if (control->port_register) {
+				control->port_register
+					(event.x.port_id, FALSE,
+					 control->port_register_arg);
+			}
+			break;
+			
+		case ClientRegistered:
+			if (control->client_register) {
+				control->client_register
+					(event.x.name, TRUE,
+					 control->client_register_arg);
+			} 
+			break;
+			
+		case ClientUnregistered:
+			if (control->client_register) {
+				control->client_register
+					(event.x.name, FALSE,
+					 control->client_register_arg);
+			}
+			break;
+			
+		case GraphReordered:
+			status = jack_handle_reorder (client, &event);
+			break;
+			
+		case PortConnected:
+		case PortDisconnected:
+			status = jack_client_handle_port_connection
+				(client, &event);
+			break;
+			
+		case BufferSizeChange:
+			jack_client_invalidate_port_buffers (client);
+			if (control->bufsize) {
+				status = control->bufsize
+					(control->nframes,
+					 control->bufsize_arg);
+			} 
+			break;
+			
+		case SampleRateChange:
+			if (control->srate) {
+				status = control->srate
+					(control->nframes,
+					 control->srate_arg);
+			}
+			break;
+			
+		case XRun:
+			if (control->xrun) {
+				status = control->xrun
+					(control->xrun_arg);
+			}
+			break;
+			
+		case AttachPortSegment:
+			jack_attach_port_segment (client, event.y.ptid);
+			break;
+			
+		case StartFreewheel:
+			jack_start_freewheel (client);
+			break;
+			
+		case StopFreewheel:
+			jack_stop_freewheel (client);
+			break;
+		}
+		
+		DEBUG ("client has dealt with the event, writing "
+		       "response on event fd");
+		
+		if (write (client->event_fd, &status, sizeof (status))
+		    != sizeof (status)) {
+			jack_error ("cannot send event response to "
+				    "engine (%s)", strerror (errno));
+			return -1;
+		}
+	}
+
 	return 0;
 }
 
+static int
+jack_client_core_wait (jack_client_t* client)
+{
+	jack_client_control_t *control = client->control;
+
+	DEBUG ("client polling on %s", client->pollmax == 2 ? 
+	       "event_fd and graph_wait_fd..." :
+	       "event_fd only");
+	
+	while (1) {
+		if (poll (client->pollfd, client->pollmax, 1000) < 0) {
+			if (errno == EINTR) {
+				continue;
+			}
+			jack_error ("poll failed in client (%s)",
+				    strerror (errno));
+			return -1;
+		}
+
+		pthread_testcancel();
+
+#ifndef JACK_USE_MACH_THREADS 
+		
+		/* get an accurate timestamp on waking from poll for a
+		 * process() cycle. 
+		 */
+		
+		if (client->graph_wait_fd >= 0
+		    && client->pollfd[WAIT_POLL_INDEX].revents & POLLIN) {
+			control->awake_at = jack_get_microseconds();
+		}
+		
+		DEBUG ("pfd[EVENT].revents = 0x%x pfd[WAIT].revents = 0x%x",
+		       client->pollfd[EVENT_POLL_INDEX].revents,
+		       client->pollfd[WAIT_POLL_INDEX].revents);
+		
+		if (client->graph_wait_fd >= 0 &&
+		    (client->pollfd[WAIT_POLL_INDEX].revents & ~POLLIN)) {
+			
+			/* our upstream "wait" connection
+			   closed, which either means that
+			   an intermediate client exited, or
+			   jackd exited, or jackd zombified
+			   us.
+			   
+			   we can discover the zombification
+			   via client->control->dead, but
+			   the other two possibilities are
+			   impossible to identify just from
+			   this situation. so we have to
+			   check what we are connected to,
+			   and act accordingly.
+			*/
+			
+			if (client->upstream_is_jackd) {
+				DEBUG ("WE DIE\n");
+				return 0;
+			} else {
+				DEBUG ("WE PUNT\n");
+				/* don't poll on the wait fd
+				 * again until we get a
+				 * GraphReordered event.
+				 */
+				
+				client->graph_wait_fd = -1;
+				client->pollmax = 1;
+			}
+		}
+#endif
+		
+		if (jack_client_process_events (client)) {
+			DEBUG ("event processing failed\n");
+			return 0;
+		}
+		
+		if (client->graph_wait_fd >= 0 &&
+		    (client->pollfd[WAIT_POLL_INDEX].revents & POLLIN)) {
+			DEBUG ("time to run process()\n");
+			break;
+		}
+	}
+
+	if (client->control->dead || client->pollfd[EVENT_POLL_INDEX].revents & ~POLLIN) {
+		DEBUG ("client appears dead or event pollfd has error status\n");
+		return -1;
+	}
+
+	return 0;
+}
+
+static int
+jack_wake_next_client (jack_client_t* client)
+{
+	char c = 0;
+
+	if (write (client->graph_next_fd, &c, sizeof (c))
+	    != sizeof (c)) {
+		DEBUG("cannot write byte to fd %d", client->graph_next_fd);
+		jack_error ("cannot continue execution of the "
+			    "processing graph (%s)",
+			    strerror(errno));
+		return -1;
+	}
+	
+	DEBUG ("client sent message to next stage by %" PRIu64
+	       ", client reading on graph_wait_fd==%d", 
+	       jack_get_microseconds(), client->graph_wait_fd);
+	
+	DEBUG("reading cleanup byte from pipe %d\n", client->graph_wait_fd);
+	
+	if ((read (client->graph_wait_fd, &c, sizeof (c))
+	     != sizeof (c))) {
+		jack_error ("cannot complete execution of the "
+			    "processing graph (%s)",
+			    strerror(errno));
+		return -1;
+	}
+	
+	return 0;
+}
+
+static jack_nframes_t 
+jack_thread_first_wait (jack_client_t* client)
+{
+	if (jack_client_core_wait (client)) {
+		return 0;
+	}
+	return client->control->nframes;
+}
+		
+jack_nframes_t
+jack_thread_wait (jack_client_t* client, int status)
+{
+	client->control->last_status = status;
+
+   /* SECTION ONE: HOUSEKEEPING/CLEANUP FROM LAST DATA PROCESSING */
+
+	/* housekeeping/cleanup after data processing */
+
+	if (status == 0 && client->control->timebase_cb) {
+		jack_call_timebase_master (client);
+	}
+	
+	/* end preemption checking */
+	CHECK_PREEMPTION (client->engine, FALSE);
+	
+	client->control->finished_at = jack_get_microseconds();
+	
+	/* wake the next client in the chain (could be the server), 
+	   and check if we were killed during the process
+	   cycle.
+	*/
+	
+	if (jack_wake_next_client (client)) {
+		DEBUG("client cannot wake next, or is dead\n");
+		return 0;
+	}
+
+	if (status || client->control->dead || !client->engine->engine_ok) {
+		return 0;
+	}
+	
+   /* SECTION TWO: WAIT FOR NEXT DATA PROCESSING TIME */
+
+	if (jack_client_core_wait (client)) {
+		return 0;
+	}
+
+   /* SECTION THREE: START NEXT DATA PROCESSING TIME */
+
+	/* Time to do data processing */
+
+	client->control->state = Running;
+	
+	/* begin preemption checking */
+	CHECK_PREEMPTION (client->engine, TRUE);
+	
+	if (client->control->sync_cb)
+		jack_call_sync_client (client);
+
+	return client->control->nframes;
+}
+
+static void *
+jack_client_thread (void *arg)
+{
+	jack_client_t *client = (jack_client_t *) arg;
+	jack_client_control_t *control = client->control;
+
+	pthread_mutex_lock (&client_lock);
+	client->thread_ok = TRUE;
+	client->thread_id = pthread_self();
+	pthread_cond_signal (&client_ready);
+	pthread_mutex_unlock (&client_lock);
+
+	control->pid = getpid();
+	control->pgrp = getpgrp();
+
+	DEBUG ("client thread is now running");
+
+	if (control->thread_init) {
+		DEBUG ("calling client thread init callback");
+		control->thread_init (control->thread_init_arg);
+	}
+
+	/* wait for first wakeup from server */
+
+	if (jack_thread_first_wait (client) == control->nframes) {
+		/* now run till we're done */
+		if (control->process) {
+			/* run process callback, then wait... ad-infinitum */
+			while (jack_thread_wait (client, 
+						 control->process (control->nframes, 
+								   control->process_arg)) ==
+			       control->nframes)
+				;
+		} else {
+			/* no process handling but still need to process events */
+			while (jack_thread_wait (client, 0) == control->nframes)
+				;
+		}
+	}
+
+	jack_client_thread_suicide (client);
+	/*NOTREACHED*/
+	return (void *) 0;
+}
+
 #ifdef JACK_USE_MACH_THREADS
 /* real-time thread : separated from the normal client thread, it will
  * communicate with the server using fast mach RPC mechanism */
@@ -1654,9 +1667,6 @@ jack_client_process_thread (void *arg)
                 
 		control->finished_at = jack_get_microseconds();
                 
-#ifdef WITH_TIMESTAMPS
-		jack_timestamp ("finished");
-#endif
 		DEBUG ("client finished processing at %Lu (elapsed = %f usecs)",
 			control->finished_at,
 			((float)(control->finished_at - control->awake_at)));
@@ -2134,6 +2144,34 @@ jack_set_port_registration_callback(jack_client_t *client,
 	return 0;
 }
 
+int
+jack_set_port_connect_callback(jack_client_t *client,
+			       JackPortConnectCallback callback,
+			       void *arg)
+{
+	if (client->control->active) {
+		jack_error ("You cannot set callbacks on an active client.");
+		return -1;
+	}
+	client->control->port_connect_arg = arg;
+	client->control->port_connect = callback;
+	return 0;
+}
+
+int
+jack_set_client_registration_callback(jack_client_t *client,
+				      JackClientRegistrationCallback callback,
+				      void *arg)
+{
+	if (client->control->active) {
+		jack_error ("You cannot set callbacks on an active client.");
+		return -1;
+	}
+	client->control->client_register_arg = arg;
+	client->control->client_register = callback;
+	return 0;
+}
+
 int
 jack_get_process_done_fd (jack_client_t *client)
 {
