@@ -16,7 +16,6 @@
     along with this program; if not, write to the Free Software 
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: port.c 998 2006-11-22 19:55:56Z joq $
 */
 
 #include <string.h>
@@ -38,7 +37,8 @@
 #include "local.h"
 
 static void    jack_generic_buffer_init(void *port_buffer,
-                                      size_t buffer_size);
+                                      size_t buffer_size,
+				      jack_nframes_t nframes);
 
 static void    jack_audio_port_mixdown (jack_port_t *port,
 					jack_nframes_t nframes);
@@ -96,216 +96,6 @@ gen_mixf (float *dest, const float *src, int length)
 
 #ifdef ARCH_X86
 
-static void
-x86_3dnow_add2f (float *dest, const float *src, int length)
-{
-	int i, n;
-	pv2sf m64p_dest = (pv2sf) dest;
-	pv2sf m64p_src = (pv2sf) src;
-
-	n = (length >> 1);
-	for (i = 0; i < n; i++)
-	{
-		asm volatile (
-			"movq %1, %%mm0\n\t" \
-			"pfadd %2, %%mm0\n\t" \
-			"movq %%mm0, %0\n\t"
-			: "=m" (m64p_dest[i])
-			: "m0" (m64p_dest[i]),
-			  "m" (m64p_src[i])
-			: "mm0", "memory");
-	}
-	if (n & 0x1)
-	{
-		asm volatile (
-			"movd %1, %%mm0\n\t" \
-			"movd %2, %%mm1\n\t" \
-			"pfadd %%mm1, %%mm0\n\t" \
-			"movd %%mm0, %0\n\t"
-			: "=m" (dest[length - 1])
-			: "m0" (dest[length - 1]),
-			  "m" (src[length - 1])
-			: "mm0", "mm1", "memory");
-	}
-	asm volatile (
-		"femms\n\t" \
-		"sfence\n\t");
-}
-
-static void
-x86_3dnow_copyf (float *dest, const float *src, int length)
-{
-	int i, n1, n2;
-	pv2sf m64p_src = (pv2sf) src;
-	pv2sf m64p_dest = (pv2sf) dest;
-
-	n1 = (length >> 4);
-	n2 = ((length & 0xf) >> 1);
-	for (i = 0; i < n1; i++)
-	{
-		asm volatile ("movq %0, %%mm0\n\t"
-			: : "m" (*m64p_src++) : "mm0", "memory");
-		asm volatile ("movq %0, %%mm1\n\t"
-			: : "m" (*m64p_src++) : "mm1", "memory");
-		asm volatile ("movq %0, %%mm2\n\t"
-			: : "m" (*m64p_src++) : "mm2", "memory");
-		asm volatile ("movq %0, %%mm3\n\t"
-			: : "m" (*m64p_src++) : "mm3", "memory");
-		asm volatile ("movq %0, %%mm4\n\t"
-			: : "m" (*m64p_src++) : "mm4", "memory");
-		asm volatile ("movq %0, %%mm5\n\t"
-			: : "m" (*m64p_src++) : "mm5", "memory");
-		asm volatile ("movq %0, %%mm6\n\t"
-			: : "m" (*m64p_src++) : "mm6", "memory");
-		asm volatile ("movq %0, %%mm7\n\t"
-			: : "m" (*m64p_src++) : "xmm7", "memory");
-
-		asm volatile ("movq %%mm0, %0\n\t"
-			: "=m" (*m64p_dest++) : : "mm0", "memory");
-		asm volatile ("movq %%mm1, %0\n\t"
-			: "=m" (*m64p_dest++) : : "mm1", "memory");
-		asm volatile ("movq %%mm2, %0\n\t"
-			: "=m" (*m64p_dest++) : : "mm2", "memory");
-		asm volatile ("movq %%mm3, %0\n\t"
-			: "=m" (*m64p_dest++) : : "mm3", "memory");
-		asm volatile ("movq %%mm4, %0\n\t"
-			: "=m" (*m64p_dest++) : : "mm4", "memory");
-		asm volatile ("movq %%mm5, %0\n\t"
-			: "=m" (*m64p_dest++) : : "mm5", "memory");
-		asm volatile ("movq %%mm6, %0\n\t"
-			: "=m" (*m64p_dest++) : : "mm6", "memory");
-		asm volatile ("movq %%mm7, %0\n\t"
-			: "=m" (*m64p_dest++) : : "mm7", "memory");
-	}
-	for (i = 0; i < n2; i++)
-	{
-		asm volatile (
-			"movq %1, %%mm0\n\t" \
-			"movq %%mm0, %0\n\t"
-			: "=m" (*m64p_dest++)
-			: "m" (*m64p_src++)
-			: "mm0", "memory");
-	}
-	if (length & 0x1)
-	{
-		asm volatile (
-			"movd %1, %%mm0\n\t" \
-			"movd %%mm0, %0\n\t"
-			: "=m" (dest[length - 1])
-			: "m" (src[length - 1])
-			: "mm0", "memory");
-	}
-	asm volatile (
-		"femms\n\t" \
-		"sfence\n\t");
-}
-
-static void
-x86_sse_copyf (float *dest, const float *src, int length)
-{
-	int i, n1, n2, si3;
-	pv4sf m128p_src = (pv4sf) src;
-	pv4sf m128p_dest = (pv4sf) dest;
-
-	n1 = (length >> 5);
-	n2 = ((length & 0x1f) >> 2);
-	si3 = (length & ~0x3);
-	for (i = 0; i < n1; i++)
-	{
-		asm volatile ("movaps %0, %%xmm0\n\t"
-			: : "m" (*m128p_src++) : "xmm0", "memory");
-		asm volatile ("movaps %0, %%xmm1\n\t"
-			: : "m" (*m128p_src++) : "xmm1", "memory");
-		asm volatile ("movaps %0, %%xmm2\n\t"
-			: : "m" (*m128p_src++) : "xmm2", "memory");
-		asm volatile ("movaps %0, %%xmm3\n\t"
-			: : "m" (*m128p_src++) : "xmm3", "memory");
-		asm volatile ("movaps %0, %%xmm4\n\t"
-			: : "m" (*m128p_src++) : "xmm4", "memory");
-		asm volatile ("movaps %0, %%xmm5\n\t"
-			: : "m" (*m128p_src++) : "xmm5", "memory");
-		asm volatile ("movaps %0, %%xmm6\n\t"
-			: : "m" (*m128p_src++) : "xmm6", "memory");
-		asm volatile ("movaps %0, %%xmm7\n\t"
-			: : "m" (*m128p_src++) : "xmm7", "memory");
-
-		asm volatile ("movaps %%xmm0, %0\n\t"
-			: "=m" (*m128p_dest++) : : "xmm0", "memory");
-		asm volatile ("movaps %%xmm1, %0\n\t"
-			: "=m" (*m128p_dest++) : : "xmm1", "memory");
-		asm volatile ("movaps %%xmm2, %0\n\t"
-			: "=m" (*m128p_dest++) : : "xmm2", "memory");
-		asm volatile ("movaps %%xmm3, %0\n\t"
-			: "=m" (*m128p_dest++) : : "xmm3", "memory");
-		asm volatile ("movaps %%xmm4, %0\n\t"
-			: "=m" (*m128p_dest++) : : "xmm4", "memory");
-		asm volatile ("movaps %%xmm5, %0\n\t"
-			: "=m" (*m128p_dest++) : : "xmm5", "memory");
-		asm volatile ("movaps %%xmm6, %0\n\t"
-			: "=m" (*m128p_dest++) : : "xmm6", "memory");
-		asm volatile ("movaps %%xmm7, %0\n\t"
-			: "=m" (*m128p_dest++) : : "xmm7", "memory");
-	}
-	for (i = 0; i < n2; i++)
-	{
-		asm volatile (
-			"movaps %1, %%xmm0\n\t" \
-			"movaps %%xmm0, %0\n\t"
-			: "=m" (*m128p_dest++)
-			: "m" (*m128p_src++)
-			: "xmm0", "memory");
-	}
-	for (i = si3; i < length; i++)
-	{
-		asm volatile (
-			"movss %1, %%xmm0\n\t" \
-			"movss %%xmm0, %0\n\t"
-			: "=m" (dest[i])
-			: "m" (src[i])
-			: "xmm0", "memory");
-	}
-}
-
-static void
-x86_sse_add2f (float *dest, const float *src, int length)
-{
-	int i, n, si2;
-	pv4sf m128p_src = (pv4sf) src;
-	pv4sf m128p_dest = (pv4sf) dest;
-
-	if (__builtin_expect(((long) src & 0xf) || ((long) dest & 0xf), 0))
-	{
-		/*fprintf(stderr, "x86_sse_add2f(): non aligned pointers!\n");*/
-		si2 = 0;
-		goto sse_nonalign;
-	}
-	si2 = (length & ~0x3);
-	n = (length >> 2);
-	for (i = 0; i < n; i++)
-	{
-		asm volatile (
-			"movaps %1, %%xmm0\n\t" \
-			"addps %2, %%xmm0\n\t" \
-			"movaps %%xmm0, %0\n\t"
-			: "=m" (m128p_dest[i])
-			: "m0" (m128p_dest[i]),
-			  "m" (m128p_src[i])
-			: "xmm0", "memory");
-	}
-sse_nonalign:
-	for (i = si2; i < length; i++)
-	{
-		asm volatile (
-			"movss %1, %%xmm0\n\t" \
-			"addss %2, %%xmm0\n\t" \
-			"movss %%xmm0, %0\n\t"
-			: "=m" (dest[i])
-			: "m0" (dest[i]),
-			  "m" (src[i])
-			: "xmm0", "memory");
-	}
-}
-
 void jack_port_set_funcs ()
 {
 	if (ARCH_X86_HAVE_SSE2(cpu_type)) {
@@ -334,6 +124,28 @@ void jack_port_set_funcs ()
 
 #endif /* USE_DYNSIMD */
 
+int
+jack_port_name_equals (jack_port_shared_t* port, const char* target)
+{
+	char buf[JACK_PORT_NAME_SIZE+1];
+
+	/* this nasty, nasty kludge is here because between 0.109.0 and 0.109.1,
+	   the ALSA audio backend had the name "ALSA", whereas as before and
+	   after it, it was called "alsa_pcm". this stops breakage for
+	   any setups that have saved "alsa_pcm" or "ALSA" in their connection
+	   state.
+	*/
+
+	if (strncmp (target, "ALSA:capture", 12) == 0 || strncmp (target, "ALSA:playback", 13) == 0) {
+		snprintf (buf, sizeof (buf), "alsa_pcm%s", target+4);
+		target = buf;
+	}
+
+	return (strcmp (port->name, target) == 0 || 
+		strcmp (port->alias1, target) == 0 || 
+		strcmp (port->alias2, target) == 0);
+}
+
 jack_port_functions_t *
 jack_get_port_functions(jack_port_type_id_t ptid)
 {
@@ -352,8 +164,8 @@ jack_get_port_functions(jack_port_type_id_t ptid)
  * Fills buffer with zeroes. For audio ports, engine->silent_buffer relies on it.
  */
 static void
-jack_generic_buffer_init(void *buffer, size_t size)
-{
+jack_generic_buffer_init(void *buffer, size_t size, jack_nframes_t nframes)
+{ 
 	memset(buffer, 0, size);
 }
 
@@ -373,7 +185,7 @@ jack_port_new (const jack_client_t *client, jack_port_id_t port_id,
 	pthread_mutex_init (&port->connection_lock, NULL);
 	port->connections = 0;
 	port->tied = NULL;
-	
+
 	if (client->control->id == port->shared->client_id) {
 			
 		/* It's our port, so initialize the pointers to port
@@ -396,7 +208,7 @@ jack_port_new (const jack_client_t *client, jack_port_id_t port_id,
 
 	port->client_segment_base =
 		(void **) &client->port_segment[ptid].attached_at;
-	
+
 	return port;
 }
 
@@ -463,26 +275,6 @@ jack_port_unregister (jack_client_t *client, jack_port_t *port)
 	return jack_client_deliver_request (client, &req);
 }
 
-int
-jack_port_lock (jack_client_t *client, jack_port_t *port)
-{
-	if (port) {
-		port->shared->locked = 1;
-		return 0;
-	}
-	return -1;
-}
-
-int
-jack_port_unlock (jack_client_t *client, jack_port_t *port)
-{
-	if (port) {
-		port->shared->locked = 0;
-		return 0;
-	}
-	return -1;
-}
-
 /* LOCAL (in-client) connection querying only */
 
 int
@@ -509,7 +301,7 @@ jack_port_connected_to (const jack_port_t *port, const char *portname)
 	for (node = port->connections; node; node = jack_slist_next (node)) {
 		jack_port_t *other_port = (jack_port_t *) node->data;
 		
-		if (strcmp (other_port->shared->name, portname) == 0) {
+		if (jack_port_name_equals (other_port->shared, portname)) {
 			ret = TRUE;
 			break;
 		}
@@ -542,7 +334,8 @@ jack_port_get_connections (const jack_port_t *port)
 				* (jack_slist_length (port->connections) + 1));
 		for (n = 0, node = port->connections; node;
 		     node = jack_slist_next (node), ++n) {
-			ret[n] = ((jack_port_t *) node->data)->shared->name;
+			jack_port_t* other =(jack_port_t *) node->data;
+			ret[n] = other->shared->name;
 		}
 		ret[n] = NULL;
 	}
@@ -665,7 +458,7 @@ jack_port_by_name_int (jack_client_t *client, const char *port_name)
 	port = &client->engine->ports[0];
 	
 	for (i = 0; i < limit; i++) {
-		if (port[i].in_use && strcmp (port[i].name, port_name) == 0) {
+		if (port[i].in_use && jack_port_name_equals (&port[i], port_name)) {
 			return jack_port_new (client, port[i].id,
 					      client->engine);
 		}
@@ -681,7 +474,7 @@ jack_port_by_name (jack_client_t *client,  const char *port_name)
 	jack_port_t* port;
 	for (node = client->ports_ext; node; node = jack_slist_next (node)) {
 		port = node->data;
-		if (strcmp (port->shared->name, port_name) == 0) {
+		if (jack_port_name_equals (port->shared, port_name)) {
 			/* Found port, return the cached structure. */
 			return port;
 		}
@@ -761,7 +554,7 @@ jack_port_get_buffer (jack_port_t *port, jack_nframes_t nframes)
 				* sizeof (jack_default_audio_sample_t)
 				* nframes;
 		port->mix_buffer = jack_pool_alloc (buffer_size);
-		port->fptr.buffer_init (port->mix_buffer, buffer_size);
+		port->fptr.buffer_init (port->mix_buffer, buffer_size, nframes);
 	}
 	port->fptr.mixdown (port, nframes);
 	return (void *) port->mix_buffer;
@@ -787,7 +580,6 @@ jack_port_tie (jack_port_t *src, jack_port_t *dst)
 
 int
 jack_port_untie (jack_port_t *port)
-
 {
 	if (port->tied == NULL) {
 		jack_error ("port \"%s\" is not tied", port->shared->name);
@@ -889,6 +681,24 @@ jack_port_name (const jack_port_t *port)
 	return port->shared->name;
 }
 
+int
+jack_port_get_aliases (const jack_port_t *port, char* const aliases[2])
+{
+	int cnt = 0;
+	
+	if (port->shared->alias1[0] != '\0') {
+		snprintf (aliases[0], JACK_CLIENT_NAME_SIZE+JACK_PORT_NAME_SIZE, "%s", port->shared->alias1);
+		cnt++;
+	}
+
+	if (port->shared->alias2[0] != '\0') {
+		snprintf (aliases[1], JACK_CLIENT_NAME_SIZE+JACK_PORT_NAME_SIZE, "%s", port->shared->alias2);
+		cnt++;
+	}
+
+	return cnt;
+}
+
 const char *
 jack_port_short_name (const jack_port_t *port)
 {
@@ -931,6 +741,35 @@ jack_port_set_name (jack_port_t *port, const char *new_name)
 	return 0;
 }
 
+int
+jack_port_set_alias (jack_port_t *port, const char *alias)
+{
+	if (port->shared->alias1[0] == '\0') {
+		snprintf (port->shared->alias1, sizeof (port->shared->alias1), "%s", alias);
+	} else if (port->shared->alias2[0] == '\0') {
+		snprintf (port->shared->alias2, sizeof (port->shared->alias2), "%s", alias);
+	} else {
+		return -1;
+	}
+
+	return 0;
+}
+
+int
+jack_port_unset_alias (jack_port_t *port, const char *alias)
+{
+	if (strcmp (port->shared->alias1, alias) == 0) {
+		port->shared->alias1[0] = '\0';
+	} else if (strcmp (port->shared->alias2, alias) == 0) {
+		port->shared->alias2[0] = '\0';
+	} else {
+		return -1;
+	}
+
+	return 0;
+}
+
+
 /* AUDIO PORT SUPPORT */
 
 static inline float f_max(float x, float a)
